/*
 * 05/28/2003
 *
 * building.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "building.h"

// ################
// Classe CBuilding
// ################


// Nombre de buildings affiches
// Doit tre remis a zero apres chaque affichage de frame
int CBuilding::NB_DISPLAY = 0;


// Dessin en cel-shading
bool CBuilding::CELSHADING = true;	


// Nombre d'instances de CBuilding
int CBuilding::NB = 0;


// ================
// Constructeur
// Type de building
// ================
CBuilding::CBuilding(id_t id, char * name, int type) : CObject(id, name)
{
	NB++;
	m_type = BUILDING;
	m_typeBuilding = type;
}


// ===========
// Destructeur
// ===========
CBuilding::~CBuilding() 
{ 
	if (glIsList(m_displayList[0])) glDeleteLists(m_displayList[0], 1);
	if (glIsList(m_displayList[1])) glDeleteLists(m_displayList[1], 1);
	NB--;
}


// ===============================================================
// Initialize la taille du building et cree les listes d'affichage
// Cette methode doit etre imperativement appele avant Draw()
// L'unite correspond a une zone de la ville
// ===============================================================
void CBuilding::SetSize(unit_t width, unit_t height, unit_t length)
{

	float w = POSZONE(width);		// Largeur du building en fonction de la taille d'une zone
	float h = POSZONE(height);		// Hauteur du building en fonction de la taille d'une zone
	float l = POSZONE(length);		// Longueur du building en fonction de la taille d'une zone

	float dec = (float)ZONE_SIZE / 6.0;	// Decalage pour que le building ne recouvre pas
										// entierement les zones correspondantes de la ville

	// Definitions des points
	m_points[0][0] = 0 + dec;		m_points[0][1] = 0.02;		m_points[0][2] = 0 + dec;
	m_points[1][0] = w - dec;		m_points[1][1] = 0.02;		m_points[1][2] = 0 + dec;
	m_points[2][0] = w - dec;		m_points[2][1] = 0.02;		m_points[2][2] = l - dec;
	m_points[3][0] = 0 + dec;		m_points[3][1] = 0.02;		m_points[3][2] = l - dec;

	m_points[4][0] = 0 + dec;		m_points[4][1] = h;			m_points[4][2] = 0 + dec;
	m_points[5][0] = w - dec;		m_points[5][1] = h;			m_points[5][2] = 0 + dec;
	m_points[6][0] = w - dec;		m_points[6][1] = h;			m_points[6][2] = l - dec;
	m_points[7][0] = 0 + dec;		m_points[7][1] = h;			m_points[7][2] = l - dec;

	CreateDisplayLists();
}


// ===========================================
// Creation des listes d'affichage du building
// ===========================================
void CBuilding::CreateDisplayLists()
{
	if (glIsList(m_displayList[0])) glDeleteLists(m_displayList[0], 1);
	m_displayList[0] = glGenLists(1);

	glNewList(m_displayList[0], GL_COMPILE);
		// Affichage du parrallepipede
		glBegin(GL_QUADS);
			
			switch (m_typeBuilding)
			{
				case RESIDENTIAL : glColor3f(0.05, 0.75, 0.05); break;
				case INDUSTRIAL : glColor3f(0.75, 0.75, 0.05); break;
				case COMMERCIAL : glColor3f(0.05, 0.05, 0.75); break;
			}
			
			// BOTTOM
			glNormal3f(0.0, -1.0, 0.0);
			glVertex3fv(m_points[0]);
			glVertex3fv(m_points[1]);
			glVertex3fv(m_points[2]);
			glVertex3fv(m_points[3]);
			// TOP
			glNormal3f(0.0, 1.0, 0.0);
			glVertex3fv(m_points[4]);
			glVertex3fv(m_points[5]);
			glVertex3fv(m_points[6]);
			glVertex3fv(m_points[7]);
			// FRONT
			glNormal3f(0.0, 0.0, 1.0);
			glVertex3fv(m_points[0]);
			glVertex3fv(m_points[1]);
			glVertex3fv(m_points[5]);
			glVertex3fv(m_points[4]);
			// RIGHT
			glNormal3f(1.0, 0.0, 0.0);
			glVertex3fv(m_points[1]);
			glVertex3fv(m_points[2]);
			glVertex3fv(m_points[6]);
			glVertex3fv(m_points[5]);
			// BACK
			glNormal3f(0.0, 0.0, -1.0);
			glVertex3fv(m_points[2]);
			glVertex3fv(m_points[3]);
			glVertex3fv(m_points[7]);
			glVertex3fv(m_points[6]);
			// LEFT
			glNormal3f(-1.0, 0.0, 0.0);
			glVertex3fv(m_points[3]);
			glVertex3fv(m_points[0]);
			glVertex3fv(m_points[4]);
			glVertex3fv(m_points[7]);

		glEnd();

	glEndList();

	if (glIsList(m_displayList[1])) glDeleteLists(m_displayList[1], 1);
	m_displayList[1] = glGenLists(1);
	float d = 0.005;

	glNewList(m_displayList[1], GL_COMPILE);

		// Affichage des traits noirs pour simuler le cell-shading
		glLineWidth(1.0);
		glBegin(GL_LINES);
		
			glColor3f(0.0,0.0,0.0);

			glVertex3f(m_points[0][0]-d, m_points[0][1]+d, m_points[0][2]-d);
			glVertex3f(m_points[1][0]+d, m_points[1][1]+d, m_points[1][2]-d);
			glVertex3f(m_points[1][0]+d, m_points[1][1]+d, m_points[1][2]-d);
			glVertex3f(m_points[2][0]+d, m_points[2][1]+d, m_points[2][2]+d);
			glVertex3f(m_points[2][0]+d, m_points[2][1]+d, m_points[2][2]+d);
			glVertex3f(m_points[3][0]-d, m_points[3][1]+d, m_points[3][2]+d);
			glVertex3f(m_points[3][0]-d, m_points[3][1]+d, m_points[3][2]+d);
			glVertex3f(m_points[0][0]-d, m_points[0][1]+d, m_points[0][2]-d);

			glVertex3f(m_points[4][0]-d, m_points[4][1], m_points[4][2]-d);
			glVertex3f(m_points[5][0]+d, m_points[5][1], m_points[5][2]-d);
			glVertex3f(m_points[5][0]+d, m_points[5][1], m_points[5][2]-d);
			glVertex3f(m_points[6][0]+d, m_points[6][1], m_points[6][2]+d);
			glVertex3f(m_points[6][0]+d, m_points[6][1], m_points[6][2]+d);
			glVertex3f(m_points[7][0]-d, m_points[7][1], m_points[7][2]+d);
			glVertex3f(m_points[7][0]-d, m_points[7][1], m_points[7][2]+d);
			glVertex3f(m_points[4][0]-d, m_points[4][1], m_points[4][2]-d);

			glVertex3f(m_points[1][0]+d, m_points[1][1]+d, m_points[1][2]-d);
			glVertex3f(m_points[5][0]+d, m_points[5][1], m_points[5][2]-d);

			glVertex3f(m_points[2][0]+d, m_points[2][1]+d, m_points[2][2]+d);
			glVertex3f(m_points[6][0]+d, m_points[6][1], m_points[6][2]+d);

			glVertex3f(m_points[3][0]-d, m_points[3][1]+d, m_points[3][2]+d);
			glVertex3f(m_points[7][0]-d, m_points[7][1], m_points[7][2]+d);

			glVertex3f(m_points[0][0]-d, m_points[0][1]+d, m_points[0][2]-d);
			glVertex3f(m_points[4][0]-d, m_points[4][1], m_points[4][2]-d);

		glEnd();
		glLineWidth(1.0);
	glEndList();	
}


// ===================
// Dessine le building
// ===================
void CBuilding::Draw(CFrustum * frustum)
{
	if (frustum == NULL || frustum->PointsInFrustum(m_vPosition.x, m_vPosition.y, m_vPosition.z, m_points, 8))
	{
		#if DEBUG_MODE > 0
		CObject::Draw(frustum);
		#endif
		NB_DISPLAY++;

		int polyMode[2];
		glGetIntegerv(GL_POLYGON_MODE, polyMode);
		glPushMatrix();
			
			glTranslatef(m_vPosition.x, m_vPosition.y, m_vPosition.z);
			
			glCallList(m_displayList[0]);
			if (CELSHADING && polyMode[0] == GL_FILL) glCallList(m_displayList[1]);
			
		glPopMatrix();
	}
}


// ==================================================
// Place le building sur la ville a la position x,z
// L'unite correspond a une zone de la ville
// positionVille correspond a la position de la ville
// ==================================================
void CBuilding::SetZonePosition(unit_t x, unit_t z, CVector positionVille)
{
	SetPosition(positionVille.x + POSZONE(x), positionVille.y, positionVille.z + POSZONE(z));
}


// ==================================================================
// Retourne le type du building (COMMERCIAL, RESIDENTIAL, INDUSTRIAL)
// ==================================================================
int CBuilding::GetTypeBuilding()
{
	return m_typeBuilding;
}


// ==================================================================
// Change le type du building (COMMERCIAL, RESIDENTIAL, INDUSTRIAL)
// ==================================================================
void CBuilding::SetTypeBuilding(int type)
{
	m_typeBuilding = type;
}
