/*
 * 05/28/2003
 *
 * bitmapGL.h
 * Repsente les buildings
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _BUILDING_H
#define _BUILDING_H

#include <GL/glut.h>
#include "object.h"
#include "zone.h"

// ################
// Classe CBuilding
// ################
class CBuilding : public CObject 
{

public:

	// Nombre de buildings affiches
	// Doit tre remis a zero apres chaque affichage de frame
	int static NB_DISPLAY;

	// Dessin en cel-shading
	bool static CELSHADING;
	
	// Nombre d'instances de CBuilding
	int static NB;

	// ================
	// Constructeur
	// Type de building
	// ================
	CBuilding(id_t id, char * name, int type);
	
	// ===========
	// Destructeur
	// ===========
	~CBuilding();

	// ===============================================================
	// Initialize la taille du building et cree les listes d'affichage
	// Cette methode doit etre imperativement appele avant Draw()
	// L'unite correspond a une zone de la ville
	// ===============================================================
	void SetSize(unit_t width, unit_t height, unit_t length);
	
	// ===================
	// Dessine le building
	// ===================
	void Draw(CFrustum * frustum);

	// ==================================================
	// Place le building sur la ville a la position x,z
	// L'unite correspond a une zone de la ville
	// positionVille correspond a la position de la ville
	// ==================================================
	void SetZonePosition(unit_t x, unit_t z, CVector positionVille);

	// ==================================================================
	// Retourne le type du building (COMMERCIAL, RESIDENTIAL, INDUSTRIAL)
	// ==================================================================
	int GetTypeBuilding();

	// ==================================================================
	// Change le type du building (COMMERCIAL, RESIDENTIAL, INDUSTRIAL)
	// ==================================================================
	void SetTypeBuilding(int type);
	
private:
	
	// ===========================================
	// Creation des listes d'affichage du building
	// ===========================================
	void CreateDisplayLists();

	// Points pour l'affichage du building
	float m_points[8][3];

	// Identifiant des listes d'affichage
	GLuint m_displayList[2];

	// Type de la zone et donc du building
	int m_typeBuilding;

};

#endif

