/*
 * 05/28/2003
 *
 * camera.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include "camera.h"

// ##############
// Classe CCamera
// ##############


// ========================
// Constructeur
// Initialise :
// position = (0, 0, 0)
// cible = (0, 1, 0.5)
// "up" vecteur = (0, 0, 1)
// ========================
CCamera::CCamera(id_t id, char * name) : CObject(id, name)
{
	m_vView		= CVector(0.0, 1.0, 0.5);
	m_vUpVector	= CVector(0.0, 0.0, 1.0);
	m_type = CAMERA;
}


// ===========
// Destructeur
// ===========
CCamera::~CCamera() { }


// ===================================================================
// Positionne la camera (modifie les vecteurs position, cible et "up")
// ===================================================================
void CCamera::PositionCamera(float positionX, float positionY, float positionZ,
				  		     float viewX,     float viewY,     float viewZ,
							 float upVectorX, float upVectorY, float upVectorZ)
{
	m_vPosition = CVector(positionX, positionY, positionZ);
	m_vView     = CVector(viewX, viewY, viewZ);
	m_vUpVector = CVector(upVectorX, upVectorY, upVectorZ);
}


// ==============================================================
// Utilise la fonction gluLookAt() pour dire a OpenGL ou regarder
// ==============================================================
void CCamera::Look()
{	
	// Donne a OpenGL la position de la camera,
	// la position de la cible et le vecteur "up"
	gluLookAt(m_vPosition.x, m_vPosition.y, m_vPosition.z,	
			  m_vView.x,	 m_vView.y,     m_vView.z,	
			  m_vUpVector.x, m_vUpVector.y, m_vUpVector.z);
}


// ===========================================================================================
// Deplace la camera ainsi que la cible vers la gauche ou la droite suivant la valeur de speed
// ===========================================================================================
void CCamera::StrafeCamera(float speed)
{
	// pour une camera fixe on ne fait rien
}


// =========================================================================================
// Deplace la camera ainsi que la cible vers l'avant ou l'arriere suivant la valeur de speed
// =========================================================================================
void CCamera::MoveCamera(float speed)
{
	// pour une camera fixe on ne fait rien
}


// ==============================================================================
// Change l'angle de vue de la camera en fonction des deplacements de la souris
// x,y correspondent a la position courante de la souris
// lx,ly correspondent a la derniere position de la souris (avant le deplacement)
// ==============================================================================
void CCamera::SetViewByMouse(int x, int y, int lx, int ly, float speed)
{
	// pour une camera fixe on ne fait rien
}


// ===================================================================
// Deplace la camera ainsi que la cible en fonction de la touche "key"
// ===================================================================
void CCamera::SetViewByKeyboard(int key, float speed)
{
	// pour une camera fixe on ne fait rien
}


// =================
// Dessine la camera
// =================
void CCamera::Draw(CFrustum * frustum)
{
		glBindTexture(GL_TEXTURE_2D, 0);
		glPushMatrix();
		glColor3f(1.0,0.0,0.0);
		glBegin(GL_LINES);
			glVertex3f(m_vPosition.x, m_vPosition.y, m_vPosition.z);
			glVertex3f(m_vView.x, m_vView.y, m_vView.z);
		glEnd();
		glTranslatef(m_vPosition.x, m_vPosition.y, m_vPosition.z);
		glutWireSphere(0.5, 10, 10);
		glPopMatrix();
}

// =======================================
// Mise a jour de la position de la camera
// =======================================
void CCamera::Update()
{
	// on ne fait rien
}
