/*
 * 05/28/2003
 *
 * camera3person.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "camera3person.h"

// #####################
// Classe CCamera3Person
// #####################


// ========================
// Constructeur
// Initialise :
// position = (0, 0, 0)
// cible = (0, 1, 0.5)
// "up" vecteur = (0, 0, 1)
// ========================
CCamera3Person::CCamera3Person(id_t id, char * name, CVehicle *target) : CCamera(id, name)
{
	m_type = CAMERA_3PERSON;
	m_target = target;

	m_distance = 1.0f;
	m_height = 1.0f;

	Update();
}


// ===========
// Destructeur
// ===========
CCamera3Person::~CCamera3Person() { }


// ==============================================================================
// Change l'angle de vue de la camera en fonction des deplacements de la souris
// x,y correspondent a la position courante de la souris
// lx,ly correspondent a la derniere position de la souris (avant le deplacement)
// ==============================================================================
void CCamera3Person::SetViewByMouse(int x, int y, int lx, int ly, float speed)
{
	// si la souris n'a pas bouge on ne fait rien
	if((y == ly) ) return;

	// On determine la distance suivant les deplacements
	// de la souris et la vitesse "speed"
	speed = (float)( (ly - y) ) / speed;

	m_distance += speed;

	if (m_distance < 0.5f) m_distance = 0.5f;
	else if (m_distance > 2.0f) m_distance = 2.0f;
}


// ===================================================================
// Deplace la camera ainsi que la cible en fonction de la touche "key"
// ===================================================================
void CCamera3Person::SetViewByKeyboard(int key, float speed)
{
	switch(key) 
	{
		case GLUT_KEY_UP:
			m_height += speed / 10.0f;
			if (m_height > 2*ZONE_SIZE) m_height = 2*ZONE_SIZE;
			break;
		case GLUT_KEY_DOWN:
			m_height -= speed / 10.0f;
			if (m_height < 0.0f) m_height = 0.0f;
			break;
		case GLUT_KEY_RIGHT:
			m_target = (CVehicle*)m_target->Next();
			break;
	}
}


// =================
// Dessine la camera
// =================
void CCamera3Person::Draw(CFrustum * frustum)
{
	glBindTexture(GL_TEXTURE_2D, 0);
	glPushMatrix();
	glColor3f(0.4,0.9,0.2);
	glBegin(GL_LINES);
		glVertex3f(m_vPosition.x, m_vPosition.y, m_vPosition.z);
		glVertex3f(m_vView.x, m_vView.y, m_vView.z);
	glEnd();
	glTranslatef(m_vPosition.x, m_vPosition.y, m_vPosition.z);
	glutWireSphere(0.5, 10, 10);
	glPopMatrix();
}


// =======================================
// Mise a jour de la position de la camera
// =======================================
void CCamera3Person::Update()
{
	m_vView.x = m_target->GetPosition().x;
	m_vView.y = m_target->GetPosition().y + ZONE_SIZE/4.0f;
	m_vView.z = m_target->GetPosition().z;

	m_vPosition.x = m_vView.x;
	m_vPosition.y = m_vView.y + ZONE_SIZE/4.0f * m_height;
	m_vPosition.z = m_vView.z;

	float d;
	float rot = m_target->GetRotation();
	if (rot >= 0 && rot <= 90) {
		d = rot / 90.0f;
		m_vPosition.x += ZONE_SIZE * m_distance * d;
		m_vPosition.z += ZONE_SIZE * m_distance * (1.0f - d);
	}
	else if (rot > 90 && rot <= 180) {
		d = (rot - 90.0f) / 90.0f;
		m_vPosition.z -= ZONE_SIZE * m_distance * d;
		m_vPosition.x += ZONE_SIZE * m_distance * (1.0f - d);
	}
	else if (rot > 180 && rot <= 270) {
		d = (rot - 180.0f) / 90.0f;
		m_vPosition.x -= ZONE_SIZE * m_distance * d;
		m_vPosition.z -= ZONE_SIZE * m_distance * (1.0f - d);
	}
	else if (rot > 270 && rot <= 360) {
		d = (rot - 270.0f) / 90.0f;
		m_vPosition.z += ZONE_SIZE * m_distance * d;
		m_vPosition.x -= ZONE_SIZE * m_distance * (1.0f - d);
	}
}


// =========================================
// Teste si la voiture est toujours presente
// =========================================
void CCamera3Person::TestTarget(CVehicle * target)
{
	if (m_target == NULL || m_target->GetZoneX() < 0) m_target = target;
}
