/*
 * 05/28/2003
 *
 * camera3person.h
 * Classe definissant une camera OpenGL a la troisieme personne
 * suivant le vaisseau spatial
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _CAMERA3PERSON_H
#define _CAMERA3PERSON_H

#include "camera.h"
#include "vehicle.h"
#include <typeinfo>
#include <iostream>

// #####################
// Classe CCamera3Person
// #####################
class CCamera3Person : public CCamera {

public:
 
	// ========================
	// Constructeur
	// Initialise :
	// position = (0, 0, 0)
	// cible = (0, 1, 0.5)
	// "up" vecteur = (0, 0, 1)
	// ========================
	CCamera3Person(id_t id, char * name, CVehicle *target);
	
	// ===========
	// Destructeur
	// ===========
	~CCamera3Person();
	
	// ==============================================================================
	// Change l'angle de vue de la camera en fonction des deplacements de la souris
	// x,y correspondent a la position courante de la souris
	// lx,ly correspondent a la derniere position de la souris (avant le deplacement)
	// ==============================================================================
	void SetViewByMouse(int x, int y, int lx, int ly, float speed);

	// ===================================================================
	// Deplace la camera ainsi que la cible en fonction de la touche "key"
	// ===================================================================
	void SetViewByKeyboard(int key, float speed);

	// =================
	// Dessine la camera
	// =================
	void Draw(CFrustum * frustum);

	// =======================================
	// Mise a jour de la position de la camera
	// =======================================
	void Update();

	// =========================================
	// Teste si la voiture est toujours presente
	// =========================================
	void TestTarget(CVehicle * target);

private:

	// Vehicule suivi par la camera
	CVehicle * m_target;

	// Distance de la camera par rapport au vehicule
	float m_distance;

	// Hauteur de la camera par rapport au vehicule
	float m_height;
};

#endif

