/*
 * 05/28/2003
 *
 * camerafree.h
 * Classe definissant une camera OpenGL libre 
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _CAMERAFREE_H
#define _CAMERAFREE_H

#include "camera.h"

// ##################
// Classe CCameraFree
// ##################
class CCameraFree : public CCamera {

public:
 
	// ========================
	// Constructeur
	// Initialise :
	// position = (0, 0, 0)
	// cible = (0, 1, 0.5)
	// "up" vecteur = (0, 0, 1)
	// ========================
	CCameraFree(id_t id, char * name);
	
	// ===========
	// Destructeur
	// ===========
	~CCameraFree();
	
	// ============================================================
	// Effectue une rotation de l'angle de vue par rapport a un axe
	// ============================================================
	void RotateView(float angle, float X, float Y, float Z);

	// ===========================================================================================
	// Deplace la camera ainsi que la cible vers la gauche ou la droite suivant la valeur de speed
	// ===========================================================================================
	void StrafeCamera(float speed);

	// =========================================================================================
	// Deplace la camera ainsi que la cible vers l'avant ou l'arriere suivant la valeur de speed
	// =========================================================================================
	void MoveCamera(float speed);

	// ==============================================================================
	// Change l'angle de vue de la camera en fonction des deplacements de la souris
	// x,y correspondent a la position courante de la souris
	// lx,ly correspondent a la derniere position de la souris (avant le deplacement)
	// ==============================================================================
	void SetViewByMouse(int x, int y, int lx, int ly, float speed);

	// ===================================================================
	// Deplace la camera ainsi que la cible en fonction de la touche "key"
	// ===================================================================
	void SetViewByKeyboard(int key, float speed);

	// =================
	// Dessine la camera
	// =================
	void Draw(CFrustum * frustum);
};

#endif

