/*
 * 05/28/2003
 *
 * cameraiso.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "cameraiso.h"

// #################
// Classe CCameraIso
// #################


// ========================
// Constructeur
// Initialise :
// position = (0, 0, 0)
// cible = (0, 1, 0.5)
// "up" vecteur = (0, 0, 1)
// ========================
CCameraIso::CCameraIso(id_t id, char * name) : CCamera(id, name)
{
	m_type = CAMERA_ISO;
}


// ===========
// Destructeur
// ===========
CCameraIso::~CCameraIso() { }


// ===========================================================================================
// Deplace la camera ainsi que la cible vers la gauche ou la droite suivant la valeur de speed
// ===========================================================================================
void CCameraIso::StrafeCamera(float speed)
{
	// On calcule le "strafe" vecteur qui est perpendiculaire au vecteur direction et "up"
	CVector vStrafe = (m_vView - m_vPosition) ^ m_vUpVector;

	// Normalisation de "strafe"
	vStrafe = CVector::Normalize(vStrafe);

	// On ajoute le "strafe" vecteur  la position de la camera
	m_vPosition.x += vStrafe.x * speed*2;
	m_vPosition.z += vStrafe.z * speed*2;

	// On ajoute le "strafe" vecteur  la position de la cible de la camera
	m_vView.x += vStrafe.x * speed*2;
	m_vView.z += vStrafe.z * speed*2;
}


// =========================================================================================
// Deplace la camera ainsi que la cible vers l'avant ou l'arriere suivant la valeur de speed
// Uniquement sur les axes X et Z, on ne fait pas monter ou descendre la camera
// =========================================================================================
void CCameraIso::MoveCamera(float speed)
{
	// Calcul le vecteur direction (la direction dans laquelle la camera regarde)
	CVector vVector = m_vView - m_vPosition;

	// Normalisation du vecteur de direction
	vVector = CVector::Normalize(vVector);

	// On ajoute le vecteur de direction  la position de la camera
	m_vPosition.x += vVector.x * speed*2;
	m_vPosition.z += vVector.z * speed*2;

	// On ajoute le vecteur de direction  la position de la cible de la camera
	m_vView.x += vVector.x * speed*2;
	m_vView.z += vVector.z * speed*2;
}


// ==============================================================================
// Deplace la camera sur la'axe des Y pour rapprocher ou eloigner la camera
// x,y correspondent a la position courante de la souris
// lx,ly correspondent a la derniere position de la souris (avant le deplacement)
// ==============================================================================
void CCameraIso::SetViewByMouse(int x, int y, int lx, int ly, float speed)
{	
	// si la souris n'a pas bouge on ne fait rien
	if((y == ly) ) return;

	// On determine la distance suivant les deplacements
	// de la souris et la vitesse "speed"
	speed = (float)( (ly - y) ) / speed * 10.0;

	// Calcul le vecteur direction (la direction dans laquelle la camera regarde)
	CVector vVector = m_vView - m_vPosition;

	// Normalisation du vecteur de direction
	vVector = CVector::Normalize(vVector);

	// On ajoute le vecteur de direction  la position de la camera
	m_vPosition.x += vVector.x * speed;
	m_vPosition.y += vVector.y * speed;
	m_vPosition.z += vVector.z * speed;

	// On ajoute le vecteur de direction  la position de la cible de la camera
	m_vView.x += vVector.x * speed;
	m_vView.y += vVector.y * speed;
	m_vView.z += vVector.z * speed;
}


// ===================================================================
// Deplace la camera ainsi que la cible en fonction de la touche "key"
// ===================================================================
void CCameraIso::SetViewByKeyboard(int key, float speed)
{
	switch(key) 
	{
		case GLUT_KEY_UP:
			MoveCamera(speed);
			break;
		case GLUT_KEY_DOWN:
			MoveCamera(-speed);
			break;
		case GLUT_KEY_RIGHT:
			StrafeCamera(speed);
			break;
		case GLUT_KEY_LEFT:
			StrafeCamera(-speed);
			break;
	}
}

// =================
// Dessine la camera
// =================
void CCameraIso::Draw(CFrustum * frustum)
{
	glBindTexture(GL_TEXTURE_2D, 0);
	glPushMatrix();
	glColor3f(1.0,0.0,1.0);
	glBegin(GL_LINES);
		glVertex3f(m_vPosition.x, m_vPosition.y, m_vPosition.z);
		glVertex3f(m_vView.x, m_vView.y, m_vView.z);
	glEnd();
	glTranslatef(m_vPosition.x, m_vPosition.y, m_vPosition.z);
	glutWireSphere(0.5, 10, 10);
	glPopMatrix();
}
