/*
 * 05/28/2003
 *
 * camerasystem.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "camerasystem.h"

// ####################
// Classe CCameraSystem
// ####################


// =======================
// Constructeur par defaut
// =======================
CCameraSystem::CCameraSystem() { }


// ===========
// Destructeur
// ===========
CCameraSystem::~CCameraSystem()
{
	m_currentCamera = NULL;
	m_list.clear();
}


// =======================================
// Ajoute une camera au systeme et definit
// celle-ci comme la camera courante
// =======================================
void CCameraSystem::Add(CCamera * c)
{
	m_list.push_back(c);
	m_currentCamera = c;
}


// =========================================
// Retourne une camera en fonction de son id
// =========================================
CCamera * CCameraSystem::Get(id_t id)
{
	for(iterCam cam = m_list.begin() ; cam != m_list.end(); ++cam)
	{
		if ((*cam)->GetID() == id)
			return *cam;
	}
	return NULL;
}


// ==========================================
// Retourne une camera en fonction de son nom
// ==========================================
CCamera * CCameraSystem::Get(char * name)
{
	for(iterCam cam = m_list.begin() ; cam != m_list.end(); ++cam)
	{
		if (strcmp((*cam)->GetName(), name) == 0)
			return *cam;
	}
	return NULL;
}


// ==========================
// Dessine toutes les cameras
// ==========================	
void CCameraSystem::DrawAll(CFrustum * frustum)
{
	for(iterCam cam = m_list.begin() ; cam != m_list.end(); ++cam)
	{
		if ((*cam) != m_currentCamera) (*cam)->Draw(frustum);
	}
}


// =================================
// Mise a jour de toutes les cameras
// =================================
void CCameraSystem::UpdateAll()
{
	for(iterCam cam = m_list.begin() ; cam != m_list.end(); ++cam)
	{
		(*cam)->Update();
	}
}


// =========================
// Change la camera courante
// =========================
void CCameraSystem::SetCurrent(CCamera * c)
{
	m_currentCamera = c;
}


// ===========================
// Retourne la camera courante
// ===========================
CCamera * CCameraSystem::GetCurrent()
{
	return m_currentCamera;
}


// ======================================================
// Retourne vrai si la camera courante est de type "free"
// ======================================================
bool CCameraSystem::IsCurrentFree()
{
	return (m_currentCamera->GetType() == CAMERA_FREE);
}


// =======================================
// Change la camera courante en prenant la 
// prochaine camera de la liste
// =======================================
void CCameraSystem::Next()
{
	for(iterCam cam = m_list.begin() ; cam != m_list.end(); ++cam)
	{
		if ((*cam)->GetID() == m_currentCamera->GetID()) {
			++cam;
			if (cam == m_list.end())
				cam = m_list.begin();
			m_currentCamera = (*cam);
			return;
		}
	}	
}
