/*
 * 05/28/2003
 *
 * camerasystem.h
 * Classe permettant la gestion des cameras 
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _CAMERASYSTEM_H
#define _CAMERASYSTEM_H

#include "camera.h"
#include <list>
#include <cstring>
using namespace std;

// Iterateur pour les listes de cameras
typedef list<CCamera*>::iterator iterCam;

// ####################
// Classe CCameraSystem
// ####################
class CCameraSystem
{
public:

	// =======================
	// Constructeur par defaut
	// =======================
	CCameraSystem();

	// ===========
	// Destructeur
	// ===========
	~CCameraSystem();

	// =======================================
	// Ajoute une camera au systeme et definit
	// celle-ci comme la camera courante
	// =======================================
	void Add(CCamera * c);
	
	// =========================================
	// Retourne une camera en fonction de son id
	// =========================================
	CCamera * Get(id_t id);
	
	// ==========================================
	// Retourne une camera en fonction de son nom
	// ==========================================
	CCamera * Get(char * name);
	
	// ==========================
	// Dessine toutes les cameras
	// ==========================
	void DrawAll(CFrustum * frustum);

	// =================================
	// Mise a jour de toutes les cameras
	// =================================
	void UpdateAll();

	// =========================
	// Change la camera courante
	// =========================
	void SetCurrent(CCamera * c);
	
	// ===========================
	// Retourne la camera courante
	// ===========================
	CCamera * GetCurrent();
	
	// ======================================================
	// Retourne vrai si la camera courante est de type "free"
	// ======================================================
	bool IsCurrentFree();
	
	// =======================================
	// Change la camera courante en prenant la 
	// prochaine camera de la liste
	// =======================================
	void Next();
	
private:

	// Liste des cameras
	list<CCamera*> m_list;

	// Camera courante
	CCamera * m_currentCamera;
};

#endif

