/*
 * 05/28/2003
 *
 * city.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "city.h"

// ############
// Classe CCity
// ############


// ============
// Constructeur
// ============
CCity::CCity() : CObject(0, "")
{
	m_type = CITY;
	
	m_width = 0;
	m_height = 0;
	m_zones = NULL;
}


// ===========
// Destructeur
// ===========
CCity::~CCity()
{
	m_buildings.clear();
	delete m_graph;
	delete[] m_zones;
}

	
// ================
// Dessine la ville
// ================
void CCity::Draw(CFrustum * frustum, int elapsedTime)
{
	// Desactive les textures
	glBindTexture(GL_TEXTURE_2D, 0);

	#if DEBUG_MODE > 0
	CObject::Draw(frustum);
	#endif

	// Affichages des zones
	glPushMatrix();
	glTranslatef(m_vPosition.x, m_vPosition.y, m_vPosition.z );

	for (int i=0 ; i<m_width ; i++)
		for (int j=0 ; j<m_height ; j++)
		{
			if (m_zones[ZONE(i,j)].GetType() == INTERSECTION)
				m_graph->GetInter(m_zones[ZONE(i,j)].GetIdInter())->Draw(frustum, elapsedTime);
			else m_zones[ZONE(i,j)].Draw(frustum);
		}

	glPopMatrix();

	// Affichage des buildings
	for(iterBui bui = m_buildings.begin() ; bui != m_buildings.end(); ++bui)
	{
		(*bui)->Draw(frustum);
	}
}


// ======================================
// Retourne le nombre de zones en largeur
// ======================================
unit_t CCity::GetWidth()
{
	return m_width;
}


// ======================================
// Retourne le nombre de zones en hauteur
// ======================================
unit_t CCity::GetHeight()
{
	return m_height;
}


// =======================================================================
// Charge les zones, les intersections et les routes  partir d'un fichier
// =======================================================================
bool CCity::LoadCity(char *filename)
{
	const int TAILLE_BUFFER = 30;
	const char *LABEL_GLOBALS = "[Globals]";
	const char *LABEL_INTERSECTIONS = "[Intersections]";
	const char *LABEL_ZONES = "[Zones]";
	const char *LABEL_ROADS = "[Roads]";
	int nb_inter, nb_zones, nb_roads;
	int x, y, type, id, d, types[4], dir, k;
	bool zones = false, roads = false, intersections = false;
	CIntersection * inter, * inter1, * inter2;
	CRoad *road;
	CRoad *emptyRoad = new CRoad(-1,-1,-1);

	char *buffer = new char[TAILLE_BUFFER];

	// Ouverture du fichier
	ifstream file;
	file.open (filename,ios::in);
	if (!file) {
		log("Cannot open file <%s>\n", filename);
		return false;
	}
	
	// Recherche du label [Globals]
	for (file >> buffer ; strcmp(buffer, LABEL_GLOBALS) != 0 && !file.eof(); file >> buffer);
	if (file.eof()) { log("Invalid file <%s> : cannot find header [Globals]\n", filename); file.close (); return false; }

	// Nom de la ville
	file >> buffer;
	strcpy(m_name, buffer);
	// Dimensions de la ville
	file >> buffer; m_width = atoi(buffer);
	file >> buffer; m_height = atoi(buffer);
	// Creation du tableau des zones
	if (m_zones != NULL) delete m_zones;
	m_zones = new CZone[m_width * m_height];
	for (x=0 ; x<m_width ; x++)
		for (y=0 ; y<m_height ; y++)
		{
			m_zones[ZONE(x,y)].x = x;
			m_zones[ZONE(x,y)].y = y;
			m_zones[ZONE(x,y)].SetRoad(emptyRoad);
		}
	// Positionnement pour centrer la ville
	SetPosition(m_vPosition.x - ((float)POSZONE(m_width)/2), m_vPosition.y, m_vPosition.z - ((float)POSZONE(m_height)/2));
	// Nombre d'intersections
	file >> buffer; nb_inter = atoi(buffer);
	m_graph = new CGraph(nb_inter);
	// Nombre de routes
	file >> buffer; nb_roads = atoi(buffer);
	// Nombre de zones
	file >> buffer; nb_zones = atoi(buffer);

	while (!file.eof())
	{
		file >> buffer;
		// Label [Intersections]
		if (strcmp(buffer, LABEL_INTERSECTIONS) == 0)
		{
			// On recupere les caracteristiques des intersections
			for (int i=0 ; i<nb_inter ; i++) {
				file >> buffer;		// id
				id = atoi(buffer);
				file >> buffer;		// position X
				x = atoi(buffer);
				file >> buffer;		// position Y
				y = atoi(buffer);
				if (x < 0 || x >= m_width || y < 0 || y >= m_height) {
					log("Invalid file <%s> : invalid intersection coordinates\n", filename);
					return false;
				}
				// Type de l'intersection pour chaque route
				file >> buffer;	
				types[0] = atoi(buffer);
				file >> buffer;	
				types[1] = atoi(buffer);
				file >> buffer;	
				types[2] = atoi(buffer);
				file >> buffer;	
				types[3] = atoi(buffer);
				m_zones[ZONE(x,y)].SetType(INTERSECTION);
				m_zones[ZONE(x,y)].SetIdInter(id);
				inter = new CIntersection(id, &(m_zones[ZONE(x,y)]));
				inter->SetTypes(types);
				m_graph->Add(inter);
			}
			intersections = true;
		}
		// Label [Roads]
		else if (strcmp(buffer, LABEL_ROADS) == 0)
		{
			if (intersections == false) {
				log("Invalid file <%s> : header roads before header ntersections\n", filename);
				return false;
			}
			else {
			for (int i=0 ; i<nb_roads ; i++) {
				file >> buffer;		// intersection 1
				x = atoi(buffer);
				file >> buffer;		// intersection 2
				y = atoi(buffer);
				m_graph->Add(x, y);	// ajoute l'arc representant la route
				// Calcul les zones correspondantes a la route
				// et on transforme ces zones en zone roads
				inter1 = m_graph->GetInter(x);
				inter2 = m_graph->GetInter(y);
				if (inter1->GetZone()->y == inter2->GetZone()->y) {
					y = inter1->GetZone()->y;
					if (inter1->GetZone()->x < inter2->GetZone()->x) {
						x = inter1->GetZone()->x;
						inter1->SetInter(RIGHT_ROADS, inter2->GetId());
						inter2->SetInter(LEFT_ROADS, inter1->GetId());
					}
					else {
						x = inter2->GetZone()->x;
						inter1->SetInter(LEFT_ROADS, inter2->GetId());
						inter2->SetInter(RIGHT_ROADS, inter1->GetId());
					}
					d = inter1->GetZone()->x - inter2->GetZone()->x;
					if (d > 0) dir = MX;
					else dir = X;
					d = abs(d);
					road = new CRoad(inter1->GetId(), inter2->GetId(), dir);
					for (k=1 ; k<d; k++) {
						m_zones[ZONE(x+k,y)].SetType(ROAD);
						m_zones[ZONE(x+k,y)].SetRoad(road);
					}
				}
				else if (inter1->GetZone()->x == inter2->GetZone()->x) {
					x = inter1->GetZone()->x;
					if (inter1->GetZone()->y < inter2->GetZone()->y) {
						y = inter1->GetZone()->y;
						inter1->SetInter(BACK_ROADS, inter2->GetId());
						inter2->SetInter(FRONT_ROADS, inter1->GetId());
					}
					else {
						y = inter2->GetZone()->y;
						inter1->SetInter(FRONT_ROADS, inter2->GetId());
						inter2->SetInter(BACK_ROADS, inter1->GetId());
					}
					d = inter1->GetZone()->y - inter2->GetZone()->y;
					if (d > 0) dir = Y;
					else dir = MY;
					d = abs(d);
					road = new CRoad(inter2->GetId(), inter1->GetId(), dir);
					for (k=1 ; k<d; k++) {
						m_zones[ZONE(x,y+k)].SetType(ROAD);
						m_zones[ZONE(x,y+k)].SetRoad(road);	
					}
				}
				road->SetCapacity(k-1);
			}
			}
			roads = true;
		}
		// Label [Zones]
		else if (strcmp(buffer, LABEL_ZONES) == 0)
		{
			for (int i=0 ; i<nb_zones ; i++) {
				file >> buffer;		// type de la zone
				type = atoi(buffer);
				file >> buffer;		// Position X
				x = atoi(buffer);
				file >> buffer;		// Position Y
				y = atoi(buffer);
				if (x < 0 || x >= m_width || y < 0 || y >= m_height) {
					log("Invalid file <%s> : invalid zone coordinates\n", filename);
				}
				else
				switch (type) 
				{
					case 1: m_zones[ZONE(x,y)].SetType(RESIDENTIAL); break;
					case 2: m_zones[ZONE(x,y)].SetType(COMMERCIAL); break;
					case 3: m_zones[ZONE(x,y)].SetType(INDUSTRIAL); break;
					default: log("Invalid file <%s> : invalid type zone\n", filename);
				}
			}
			zones = true;
		}
	}
	file.close();
	delete[] buffer;

	// Si il manque un label: message d'erreur
	if (!(roads && intersections && zones)) {
		if (!roads) log("Invalid file <%s> : cannot find header [Roads]\n", filename);
		if (!intersections) log("Invalid file <%s> : cannot find header [Intersections]\n", filename);
		if (!zones) log("Invalid file <%s> : cannot find header [Zones]\n", filename);
		return false;
	}
	m_graph->DisplayToString();

	return true;
}


// ====================================================
// Charge les elements graphiques  partir d'un fichier
// ====================================================
bool CCity::LoadGraphics(char *filename)
{
	const int TAILLE_BUFFER = 30;
	const char *LABEL_GLOBALS = "[Globals]";
	const char *LABEL_BUILDINGS = "[Buildings]";
	int nb_buildings = 0;
	int x, y, type, w, h, l;

	char *buffer = new char[TAILLE_BUFFER];

	// Ouverture du fichier
	ifstream file;
	file.open (filename,ios::in);
	if (!file) {
		log("Cannot open file <%s>\n", filename);
		return false;
	}

	// Recherche du label [Globals]
	for (file >> buffer ; strcmp(buffer, LABEL_GLOBALS) != 0 && !file.eof(); file >> buffer);
	if (file.eof()) { log("Invalid file <%s> : cannot find header [Globals]\n", filename); file.close (); return false; }

	// Nombre de buildings
	file >> buffer; nb_buildings = atoi(buffer);

	while (!file.eof())
	{
		file >> buffer;
		// Label [Intersections]
		if (strcmp(buffer, LABEL_BUILDINGS) == 0)
		{
			for (int i=0 ; i<nb_buildings ; i++) {
				file >> buffer;
				type = atoi(buffer);	// Type du building
				file >> buffer;
				x = atoi(buffer);		// Position X
				file >> buffer;
				y = atoi(buffer);		// Position Y
				file >> buffer;
				w = atoi(buffer);		// Largeur
				file >> buffer;
				h = atoi(buffer);		// Hauteur
				file >> buffer;
				l = atoi(buffer);		// Longueur
				if (x < 0 || x >= m_width || y < 0 || y >= m_height
					|| w < 0 || h < 0 || l < 0) {
					log("Invalid file <%s> : invalid building size or position\n", filename);
					return false;
				}
				switch (type) 
				{
					case 1: type = RESIDENTIAL; break;
					case 2: type = COMMERCIAL; break;
					case 3: type = INDUSTRIAL; break;
				}
				// Creation du building
				CBuilding * building = new CBuilding(0,"",type);
				building->SetZonePosition(x, y,m_vPosition);
				building->SetSize(w,h,l);
				// Ajout du building a la liste
				m_buildings.push_back(building);
			}
		}
	}

	file.close();
	delete[] buffer;

	return true;
}


// ==============================
// Retourne le graphe de la ville
// ==============================
CGraph * CCity::GetGraph()
{
	return m_graph;
}


// =============================
// Retourne une zone de la ville
// =============================
CZone CCity::GetZone(int i)
{
	return m_zones[i];
}


// =============================
// Retourne une zone de la ville
// =============================
CZone CCity::GetZone(int i, int j)
{
	return m_zones[ZONE(i,j)];
}


