/*
 * 05/28/2003
 *
 * city.h
 * Reprsente une ville
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _CITY_H
#define _CITY_H

#include "error.h"
#include "object.h"
#include "zone.h"
#include "frustum.h"
#include "building.h"
#include "graph.h"
#include "road.h"
#include <math.h>
#include <list>
#include <iostream>
#include <fstream>
using namespace std;

// Iterateur pour les listes de buildings
typedef list<CBuilding*>::iterator iterBui;

// ############
// Classe CCity
// ############
class CCity: public CObject
{
public:

	// Macros permettant de retrouver dans le tableau 1D une coordonnees 2D
	#define ZONE(X,Y) ((Y) * m_width + (X))

	// ============
	// Constructeur
	// ============
	CCity();

	// ===========
	// Destructeur
	// ===========
	~CCity();

	// ================
	// Dessine la ville
	// ================
	void Draw(CFrustum * frustum, int elapsedTime);

	// ======================================
	// Retourne le nombre de zones en largeur
	// ======================================
	unit_t GetWidth();

	// ======================================
	// Retourne le nombre de zones en hauteur
	// ======================================
	unit_t GetHeight();

	// =======================================================================
	// Charge les zones, les intersections et les routes  partir d'un fichier
	// =======================================================================
	bool LoadCity(char *filename);

	// ====================================================
	// Charge les elements graphiques  partir d'un fichier
	// ====================================================
	bool LoadGraphics(char *filename);

	// ==============================
	// Retourne le graphe de la ville
	// ==============================
	CGraph * GetGraph();

	// =============================
	// Retourne une zone de la ville
	// =============================
	CZone GetZone(int i);

	// =============================
	// Retourne une zone de la ville
	// =============================
	CZone GetZone(int i, int j);

private:

	// Tableau contenant les zones de la ville
	CZone *m_zones;

	// Liste des buildings
	list<CBuilding*> m_buildings;

	// Nombre de zones en largeur
	unit_t m_width;

	// Nombre de zones en hauteur
	unit_t m_height;

	// Graphe de la ville
	CGraph *m_graph;

};

#endif

