/*
 * 05/28/2003
 *
 * frustum.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "frustum.h"

// ###############
// Classe CFrustum
// ###############

// Les 6 plans dur frustum
enum FrustumSide
{
	RIGHT	= 0,		// The RIGHT side of the frustum
	LEFT	= 1,		// The LEFT	 side of the frustum
	BOTTOM	= 2,		// The BOTTOM side of the frustum
	TOP		= 3,		// The TOP side of the frustum
	BACK	= 4,		// The BACK	side of the frustum
	FRONT	= 5			// The FRONT side of the frustum
}; 

// Coordonnees d'un plan
enum PlaneData
{
	A = 0,				// The X value of the plane's normal
	B = 1,				// The Y value of the plane's normal
	C = 2,				// The Z value of the plane's normal
	D = 3				// The distance the plane is from the origin
};


// ==================================
// Normalisation d'un plan du frustum
// ==================================
void CFrustum::NormalizePlane(float frustum[6][4], int side)
{
	// On calcule la longueur (magnitude) de la normale(A,B,C) du plan
	float magnitude = (float)sqrt( frustum[side][A] * frustum[side][A] + 
								   frustum[side][B] * frustum[side][B] + 
								   frustum[side][C] * frustum[side][C] );

	// On normalize le plan on divisant par la longeur
	frustum[side][A] /= magnitude;
	frustum[side][B] /= magnitude;
	frustum[side][C] /= magnitude;
	frustum[side][D] /= magnitude; 
}


// =============================================
// Calcul le frustum de vue
// Doit etre appele a chaque mouvement de camera
// =============================================
void CFrustum::CalculateFrustum()
{    
	float   proj[16];								// This will hold our projection matrix
	float   modl[16];								// This will hold our modelview matrix
	float   clip[16];								// This will hold the clipping planes

	// Recupere la matrix de projection
	glGetFloatv( GL_PROJECTION_MATRIX, proj );

	// Recupere la matrix de modelisation
	glGetFloatv( GL_MODELVIEW_MATRIX, modl );

	// On multiplie les 2 matrices entre elles
	clip[ 0] = modl[ 0] * proj[ 0] + modl[ 1] * proj[ 4] + modl[ 2] * proj[ 8] + modl[ 3] * proj[12];
	clip[ 1] = modl[ 0] * proj[ 1] + modl[ 1] * proj[ 5] + modl[ 2] * proj[ 9] + modl[ 3] * proj[13];
	clip[ 2] = modl[ 0] * proj[ 2] + modl[ 1] * proj[ 6] + modl[ 2] * proj[10] + modl[ 3] * proj[14];
	clip[ 3] = modl[ 0] * proj[ 3] + modl[ 1] * proj[ 7] + modl[ 2] * proj[11] + modl[ 3] * proj[15];

	clip[ 4] = modl[ 4] * proj[ 0] + modl[ 5] * proj[ 4] + modl[ 6] * proj[ 8] + modl[ 7] * proj[12];
	clip[ 5] = modl[ 4] * proj[ 1] + modl[ 5] * proj[ 5] + modl[ 6] * proj[ 9] + modl[ 7] * proj[13];
	clip[ 6] = modl[ 4] * proj[ 2] + modl[ 5] * proj[ 6] + modl[ 6] * proj[10] + modl[ 7] * proj[14];
	clip[ 7] = modl[ 4] * proj[ 3] + modl[ 5] * proj[ 7] + modl[ 6] * proj[11] + modl[ 7] * proj[15];

	clip[ 8] = modl[ 8] * proj[ 0] + modl[ 9] * proj[ 4] + modl[10] * proj[ 8] + modl[11] * proj[12];
	clip[ 9] = modl[ 8] * proj[ 1] + modl[ 9] * proj[ 5] + modl[10] * proj[ 9] + modl[11] * proj[13];
	clip[10] = modl[ 8] * proj[ 2] + modl[ 9] * proj[ 6] + modl[10] * proj[10] + modl[11] * proj[14];
	clip[11] = modl[ 8] * proj[ 3] + modl[ 9] * proj[ 7] + modl[10] * proj[11] + modl[11] * proj[15];

	clip[12] = modl[12] * proj[ 0] + modl[13] * proj[ 4] + modl[14] * proj[ 8] + modl[15] * proj[12];
	clip[13] = modl[12] * proj[ 1] + modl[13] * proj[ 5] + modl[14] * proj[ 9] + modl[15] * proj[13];
	clip[14] = modl[12] * proj[ 2] + modl[13] * proj[ 6] + modl[14] * proj[10] + modl[15] * proj[14];
	clip[15] = modl[12] * proj[ 3] + modl[13] * proj[ 7] + modl[14] * proj[11] + modl[15] * proj[15];
	

	// Extrait le plan RIGHT du frustum
	m_frustum[RIGHT][A] = clip[ 3] - clip[ 0];
	m_frustum[RIGHT][B] = clip[ 7] - clip[ 4];
	m_frustum[RIGHT][C] = clip[11] - clip[ 8];
	m_frustum[RIGHT][D] = clip[15] - clip[12];

	// Normalize le plan RIGHT
	NormalizePlane(m_frustum, RIGHT);

	// Extrait le plan LEFT du frustum
	m_frustum[LEFT][A] = clip[ 3] + clip[ 0];
	m_frustum[LEFT][B] = clip[ 7] + clip[ 4];
	m_frustum[LEFT][C] = clip[11] + clip[ 8];
	m_frustum[LEFT][D] = clip[15] + clip[12];

	// Normalize the LEFT side
	NormalizePlane(m_frustum, LEFT);

	// Extrait le plan BOTTOM du frustum
	m_frustum[BOTTOM][A] = clip[ 3] + clip[ 1];
	m_frustum[BOTTOM][B] = clip[ 7] + clip[ 5];
	m_frustum[BOTTOM][C] = clip[11] + clip[ 9];
	m_frustum[BOTTOM][D] = clip[15] + clip[13];

	// Normalize the BOTTOM side
	NormalizePlane(m_frustum, BOTTOM);

	// Extrait le plan TOP du frustum
	m_frustum[TOP][A] = clip[ 3] - clip[ 1];
	m_frustum[TOP][B] = clip[ 7] - clip[ 5];
	m_frustum[TOP][C] = clip[11] - clip[ 9];
	m_frustum[TOP][D] = clip[15] - clip[13];

	// Normalize the TOP side
	NormalizePlane(m_frustum, TOP);

	// Extrait le plan BACK du frustum
	m_frustum[BACK][A] = clip[ 3] - clip[ 2];
	m_frustum[BACK][B] = clip[ 7] - clip[ 6];
	m_frustum[BACK][C] = clip[11] - clip[10];
	m_frustum[BACK][D] = clip[15] - clip[14];

	// Normalize the BACK side
	NormalizePlane(m_frustum, BACK);

	// Extrait le plan FRONT du frustum
	m_frustum[FRONT][A] = clip[ 3] + clip[ 2];
	m_frustum[FRONT][B] = clip[ 7] + clip[ 6];
	m_frustum[FRONT][C] = clip[11] + clip[10];
	m_frustum[FRONT][D] = clip[15] + clip[14];

	/// Normalize the FRONT side
	NormalizePlane(m_frustum, FRONT);
}


// ===============================================
// Teste si un point 3D est dans le frustum de vue
// ===============================================
bool CFrustum::PointInFrustum( float x, float y, float z )
{
	// Boucle pour tous les plans
	for(int i = 0; i < 6; i++ )
	{
		// Teste si on est dans le plan du frustum
		if(m_frustum[i][A] * x + m_frustum[i][B] * y + m_frustum[i][C] * z + m_frustum[i][D] <= 0)
		{
			return false;
		}
	}
	// Retourne true si le point est dans tous les plans
	return true;
}


// ===============================================================
// Teste si une sphere (en fonction de son centre et de son rayon)
// est dans le frustum de vue
// ===============================================================
bool CFrustum::SphereInFrustum( float x, float y, float z, float radius )
{
	// Boucle pour tous les plans
	for(int i = 0; i < 6; i++ )	
	{
		if( m_frustum[i][A] * x + m_frustum[i][B] * y + m_frustum[i][C] * z + m_frustum[i][D] <= -radius )
		{
			// La distance est plus grand que le rayon donc la sphere est en dehors du frustum
			return false;
		}
	// La sphere est dans le frustum
	}
	return true;
}


// ==============================================================
// Teste si un cube (en fonction de son centre et de sa taille/2)
// est dans le frustum de vue
// ==============================================================
bool CFrustum::CubeInFrustum( float x, float y, float z, float size )
{
	for(int i = 0; i < 6; i++ )
	{
		if(m_frustum[i][A] * (x - size) + m_frustum[i][B] * (y - size) + m_frustum[i][C] * (z - size) + m_frustum[i][D] > 0)
		   continue;
		if(m_frustum[i][A] * (x + size) + m_frustum[i][B] * (y - size) + m_frustum[i][C] * (z - size) + m_frustum[i][D] > 0)
		   continue;
		if(m_frustum[i][A] * (x - size) + m_frustum[i][B] * (y + size) + m_frustum[i][C] * (z - size) + m_frustum[i][D] > 0)
		   continue;
		if(m_frustum[i][A] * (x + size) + m_frustum[i][B] * (y + size) + m_frustum[i][C] * (z - size) + m_frustum[i][D] > 0)
		   continue;
		if(m_frustum[i][A] * (x - size) + m_frustum[i][B] * (y - size) + m_frustum[i][C] * (z + size) + m_frustum[i][D] > 0)
		   continue;
		if(m_frustum[i][A] * (x + size) + m_frustum[i][B] * (y - size) + m_frustum[i][C] * (z + size) + m_frustum[i][D] > 0)
		   continue;
		if(m_frustum[i][A] * (x - size) + m_frustum[i][B] * (y + size) + m_frustum[i][C] * (z + size) + m_frustum[i][D] > 0)
		   continue;
		if(m_frustum[i][A] * (x + size) + m_frustum[i][B] * (y + size) + m_frustum[i][C] * (z + size) + m_frustum[i][D] > 0)
		   continue;

		return false;
	}
	return true;
}


// ===================================================
// Teste si un des points du tableau p de dimension nb
// est dans le frustum
// ===================================================
bool CFrustum::PointsInFrustum( float x, float y, float z, float p[][3], int nb )
{
	for(int i = 0; i < 6; i++ )
	{
		int j;
		for (j = 0; j < nb; j++ ) {
			if(m_frustum[i][A] * (p[j][0]+x) + m_frustum[i][B] * (p[j][1]+y) + m_frustum[i][C] * (p[j][2]+z) + m_frustum[i][D] > 0)
				break;
		}
		if (j == nb)
			return false;
	}
	return true;
}

