/*
 * 05/28/2003
 *
 * frustum.h
 * Classe permettant de tester si des objets sont
 * dans la ligne de vue
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _FRUSTUM_H
#define _FRUSTUM_H

#include <GL/glut.h>
#include <math.h>

// ###############
// Classe CFrustum
// ###############
class CFrustum 
{
public:

	// =============================================
	// Calcul le frustum de vue
	// Doit etre appele a chaque mouvement de camera
	// =============================================
	void CalculateFrustum();

	// ===============================================
	// Teste si un point 3D est dans le frustum de vue
	// ===============================================
	bool PointInFrustum(float x, float y, float z);

	// ===============================================================
	// Teste si une sphere (en fonction de son centre et de son rayon)
	// est dans le frustum de vue
	// ===============================================================
	bool SphereInFrustum(float x, float y, float z, float radius);

	// ==============================================================
	// Teste si un cube (en fonction de son centre et de sa taille/2)
	// est dans le frustum de vue
	// ==============================================================
	bool CubeInFrustum( float x, float y, float z, float size );

	// ===================================================
	// Teste si un des points du tableau p de dimension nb
	// est dans le frustum
	// ===================================================
	bool PointsInFrustum( float x, float y, float z, float p[][3], int nb);

private:
	// ==================================
	// Normalisation d'un plan du frustum
	// ==================================
	void static NormalizePlane(float frustum[6][4], int side);

	// Contient les A,B,C (normale) et D coordonees des 6 plans du frustum
	float m_frustum[6][4];
};

#endif

