/*
 * 05/28/2003
 *
 * global.h
 * Constantes ou macros
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _GLOBAL_H
#define _GLOBAL_H

// ==========================================
// Definitions de constantes et macros utiles
// ==========================================

// Nom de l'application
#define APP_NAME "SimTraffic"

// Version
#define RELEASE "1.0"

// Build
#define BUILD "b030520"

// Mode debug
#define DEBUG_MODE 0

// Fichier "log"
#define LOG_FILE "log.txt"

// Rpertoire des villes
#define MAPS_DIR "maps/"

// Code ASCII de la touche escape
const unsigned char ESC = 27;

// Activation du mip-mapping
#define MIPMAPPING false

// Type des identifiants
typedef unsigned int id_t;

// Taille d'une zone de la ville
const float ZONE_SIZE = 6.0f;

// Macros permettant d'une convertir une unite  
// sur la ville en position reelle
#define POSZONE(X) ((float)(X) * ZONE_SIZE)

// Type servant d'unite
typedef unsigned char unit_t;

// Types de zones
enum TypeZone
{
	FREE,
	ROAD,
	INTERSECTION,
	RESIDENTIAL,
	COMMERCIAL,
	INDUSTRIAL
};

const int BACK_ROADS = 0;
const int RIGHT_ROADS = 1;
const int FRONT_ROADS = 2;
const int LEFT_ROADS = 3;
const int NO_ROADS = -1;

// Intersection transparente
const int TRANSP = -1;

// Directions logiques
const int Y = BACK_ROADS;
const int X = RIGHT_ROADS;
const int MY = FRONT_ROADS;
const int MX = LEFT_ROADS;

// Type de franchissement d'une intersection
const int RIGHT_TURN = 1;
const int LEFT_TURN = 3;
const int NO_TURN = 2;

// Distance de securite entre les vehicules
const float SECURE_DIST = (ZONE_SIZE / 6.0f);

// Sens de circulation (positif ou negatif)
const int P = 0;
const int N = 1;

// Macros permettant de savoir le sens de circulation P ou N
// en fonction de la direction d'un vehicule
#define SENS(D) ((D == X || D == Y) ? P : N)

// Longueur d'une voiture
#define CAR_LENGTH (ZONE_SIZE / 2.5f)

// Flot engendre par une voiture
const float CAR_STREAM = ((CAR_LENGTH + SECURE_DIST*1.15f) / ZONE_SIZE);

#endif

