/*
 * 05/28/2003
 *
 * graph.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "graph.h"

// #############
// Classe CGraph
// #############


// ======================================
// Constructeur
// Nombre maxi de sommets du graphe: size
// ======================================
CGraph::CGraph(int size)
{
	m_size = size;
	m_inter = new CIntersection*[m_size];
	m_arcs = new bool[m_size * m_size];

	int i;
	for (i=0 ; i<m_size ; i++)
		for (int j=0 ; j<m_size ; j++)
			m_arcs[i*m_size+j] = false;

	for (i=0 ; i<m_size ; i++)
		m_inter[i] = NULL;

}


// ===========
// Destructeur
// ===========
CGraph::~CGraph()
{
	delete[] m_arcs;
	for (int i=0 ; i<m_size ; i++)
		delete m_inter[i];
	delete[] m_inter;
}


// ============================================================================
// Ajout d'une intersection
// Attention les ids des intersections doivent tre compris entre O et m_size-1
// ============================================================================
void CGraph::Add(CIntersection * x)
{
	m_inter[x->GetId()] = x;
}


// ============================================================
// Ajout d'un arc entre les intersections d'identifiants x et y
// ============================================================
void CGraph::Add(int x, int y)
{
	m_arcs[x*m_size+y] = true;
	m_arcs[y*m_size+x] = true;
}


// =========================
// Nombre d'arcs d'un sommet
// =========================
int CGraph::Degree(int x)
{
	int res = 0;
	
	for (int i=0 ; i<m_size ; i++)
		if (m_arcs[i*m_size+x] == true)
			res++;
	return res;
}


// =============================
// Teste l'existence d'un sommet
// =============================
bool CGraph::Exist(int x)
{
	if (m_inter[x] == NULL) return false;
	else return true;
}


// ==========================
// Teste l'existence d'un arc
// ==========================
bool CGraph::Exist(int x, int y)
{
	return (m_arcs[x*m_size+y] == true);
}


// ===============================
// Affiche la matrice d'adjacences
// ===============================
void CGraph::DisplayToString()
{
	printf("    ");
	
	int i;
	for (i=0 ; i<m_size ; i++)
		if (m_inter[i] != NULL)
			printf("%2d ", i);
	printf("\n");

	for (i=0 ; i<m_size ; i++)
		if (m_inter[i] != NULL)
		{
			printf("%2d| ",i);
			for (int j=0 ; j<m_size ; j++)
			{
				if (m_inter[j] != NULL)
					printf("%2d ",Exist(i,j));
			}
			printf("\n");
		}
}


// ======================================================
// Retourne l'intersection en fonction de son identifiant
// ======================================================
CIntersection * CGraph::GetInter(int id)
{
	return m_inter[id];
}
