/*
 * 05/28/2003
 *
 * graph.h
 * Graphe d'intersections
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _GRAPH_H
#define _GRAPH_H

#include <cstdio>
#include "intersection.h"

// #############
// Classe CGraph
// #############
class CGraph
{
public:

	// ======================================
	// Constructeur
	// Nombre maxi de sommets du graphe: size
	// ======================================
	CGraph(int size);

	// ===========
	// Destructeur
	// ===========
	~CGraph();

	// ============================================================================
	// Ajout d'une intersection
	// Attention les ids des intersections doivent tre compris entre O et m_size-1
	// ============================================================================
	void Add(CIntersection * x);

	// ============================================================
	// Ajout d'un arc entre les intersections d'identifiants x et y
	// ============================================================
	void Add(int x, int y);

	// =========================
	// Nombre d'arcs d'un sommet
	// =========================
	int Degree(int x);

	// =============================
	// Teste l'existence d'un sommet
	// =============================
	bool Exist(int x);

	// ==========================
	// Teste l'existence d'un arc
	// ==========================
	bool Exist(int x, int y);

	// ===============================
	// Affiche la matrice d'adjacences
	// ===============================
	void DisplayToString();

	// ======================================================
	// Retourne l'intersection en fonction de son identifiant
	// ======================================================
	CIntersection * GetInter(int id);

private:

	// Taille du graphe
	int m_size;

	// Tableau de pointeur sur les intersections
	CIntersection **m_inter;

	// Matrice d'adjacences representant les arcs
	bool *m_arcs;

};

#endif

