/*
 * 05/28/2003
 *
 * intersection.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "intersection.h"

// ####################
// Classe CIntersection
// ####################


// =========================================================
// Constructeur
// id: identifiant de l'intersection
// zone: pointeur sur la zone correspondant a l'intersection
// =========================================================
CIntersection::CIntersection(id_t id, CZone *zone)
{
	m_zone = zone;
	m_zone->SetType(INTERSECTION);
	m_id = id;
	m_roads[0] = NO_ROADS; m_roads[1] = NO_ROADS;
	m_roads[2] = NO_ROADS; m_roads[3] = NO_ROADS;
}


// ==================================================
// Retourne le pointeur sur la zone de l'intersection
// ==================================================
CZone * CIntersection::GetZone()
{
	return m_zone;
}


// ========================================
// Retourne l'identifiant de l'intersection
// ========================================
id_t CIntersection::GetId()
{
	return m_id;
}


// ===========================================================
// Place dans le tableau a la position i, l'intersection inter
// qui representera la route m_id,inter
// ===========================================================
void CIntersection::SetInter(int i, id_t inter)
{
	m_roads[i] = inter;
}


// ==================================================================================
// Retourne l'identifiant de l'intersection se trouvant a droite, gauche, haut ou bas
// ==================================================================================
id_t CIntersection::GetInter(int i)
{
	return m_roads[i];
}


// ======================================================
// Retourne la route (intersection) correspondant a i 
// i = BACK_ROADS, RIGHT_ROADS, FRONT_ROADS ou LEFT_ROADS
// ======================================================
int CIntersection::GetRoad(int i)
{
	return m_roads[i];
}


// =========================================================================
// Retourne la position (BACK_ROADS, RIGHT_ROADS, FRONT_ROADS ou LEFT_ROADS)
// de la route d'identifiant id
// =========================================================================
int CIntersection::GetPositionRoad(int id)
{
	for (int i=0 ; i<4 ; i++)
		if (m_roads[i] == id) return i;

	return -1;
}


// ===============================================
// Change le type d'intersection pour chaque route
// ===============================================
void CIntersection::SetTypes(int types[4])
{
	if (types[0] == RED || types[0] == GREEN || types[0] == ORANGE)
	{
		m_types[0] = new CTrafficLight(types[0]);
	}
	else m_types[0] = new CObject();
	if (types[1] == RED || types[1] == GREEN || types[1] == ORANGE)
	{
		m_types[1] = new CTrafficLight(types[1]);
	}
	else m_types[1] = new CObject();
	if (types[2] == RED || types[2] == GREEN || types[2] == ORANGE)
	{
		m_types[2] = new CTrafficLight(types[2]);
	}
	else m_types[2] = new CObject();
	if (types[3] == RED || types[3] == GREEN || types[3] == ORANGE)
	{
		m_types[3] = new CTrafficLight(types[3]);
	}
	else m_types[3] = new CObject();
}


// ======================
// Dessine l'intersection
// ======================
void CIntersection::Draw(CFrustum * frustum, int elapsedTime)
{
	glPushMatrix();
	glTranslatef(POSZONE(m_zone->x), 0.0, POSZONE(m_zone->y));

	glColor3f(0.5, 0.5, 0.5);
	
	glBegin(GL_QUADS);
		glVertex3f(0.0, 0.0, 0.0);
		glVertex3f(ZONE_SIZE, 0.0, 0.0);
		glVertex3f(ZONE_SIZE, 0.0, ZONE_SIZE);
		glVertex3f(0.0, 0.0, ZONE_SIZE);
	glEnd();

	// Dessine les feux
	if (m_types[RIGHT_ROADS]->GetType() == TRAFFIC_LIGHT)
	{
		((CTrafficLight *)m_types[RIGHT_ROADS])->Change(elapsedTime);
		
		glPushMatrix();
		glTranslatef(ZONE_SIZE, 0.001, 0.0);
		glRotatef(0.0, 0.0, 1.0, 0.0);
		((CTrafficLight *)m_types[RIGHT_ROADS])->Draw(frustum);
		glPopMatrix();
	}

	if (m_types[FRONT_ROADS]->GetType() == TRAFFIC_LIGHT)
	{
		((CTrafficLight *)m_types[FRONT_ROADS])->Change(elapsedTime);
		
		glPushMatrix();
		glTranslatef(0.0, 0.001, 0.0);
		glRotatef(90.0, 0.0, 1.0, 0.0);
		((CTrafficLight *)m_types[FRONT_ROADS])->Draw(frustum);
		glPopMatrix();
	}

	if (m_types[LEFT_ROADS]->GetType() == TRAFFIC_LIGHT)
	{
		((CTrafficLight *)m_types[LEFT_ROADS])->Change(elapsedTime);
		
		glPushMatrix();
		glTranslatef(0.0, 0.001, ZONE_SIZE);
		glRotatef(180.0, 0.0, 1.0, 0.0);
		((CTrafficLight *)m_types[LEFT_ROADS])->Draw(frustum);
		glPopMatrix();
	}

	if (m_types[BACK_ROADS]->GetType() == TRAFFIC_LIGHT)
	{
		((CTrafficLight *)m_types[BACK_ROADS])->Change(elapsedTime);
		
		glPushMatrix();
		glTranslatef(ZONE_SIZE, 0.001, ZONE_SIZE);
		glRotatef(270.0, 0.0, 1.0, 0.0);
		((CTrafficLight *)m_types[BACK_ROADS])->Draw(frustum);
		glPopMatrix();
	}

	glPopMatrix();
}


// ======================================================
// Retourne le type d'intersection correspondant a i 
// i = BACK_ROADS, RIGHT_ROADS, FRONT_ROADS ou LEFT_ROADS
// ======================================================
int CIntersection::GetType(int i)
{
	int res;
	switch (m_types[i]->GetType())
	{
		case TRAFFIC_LIGHT:
			res = ((CTrafficLight *)m_types[i])->GetLight();
			break;
		default:
			res = TRANSP;
	}
	return res;
}
