/*
 * 05/28/2003
 *
 * intersection.h
 * Reprsente une intersection
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _INTERSECTION_H
#define _INTERSECTION_H

#include "zone.h"
#include "object.h"
#include "frustum.h"
#include "trafficlight.h"


// ####################
// Classe CIntersection
// ####################
class CIntersection
{
public:
	
	// =========================================================
	// Constructeur
	// id: identifiant de l'intersection
	// zone: pointeur sur la zone correspondant a l'intersection
	// =========================================================
	CIntersection(id_t id, CZone *zone);

	// ==================================================
	// Retourne le pointeur sur la zone de l'intersection
	// ==================================================
	CZone * GetZone();

	// ========================================
	// Retourne l'identifiant de l'intersection
	// ========================================
	id_t GetId();

	// ===========================================================
	// Place dans le tableau a la position i, l'intersection inter
	// qui representera la route m_id,inter
	// ===========================================================
	void SetInter(int i, id_t inter);

	// ==================================================================================
	// Retourne l'identifiant de l'intersection se trouvant a droite, gauche, haut ou bas
	// ==================================================================================
	id_t GetInter(int i);

	// ======================================================
	// Retourne la route (intersection) correspondant a i 
	// i = BACK_ROADS, RIGHT_ROADS, FRONT_ROADS ou LEFT_ROADS
	// ======================================================
	int GetRoad(int i);

	// =========================================================================
	// Retourne la position (BACK_ROADS, RIGHT_ROADS, FRONT_ROADS ou LEFT_ROADS)
	// de la route d'identifiant id
	// =========================================================================
	int GetPositionRoad(int id);

	// ===============================================
	// Change le type d'intersection pour chaque route
	// ===============================================
	void SetTypes(int types[4]);

	// ======================================================
	// Retourne le type d'intersection correspondant a i 
	// i = BACK_ROADS, RIGHT_ROADS, FRONT_ROADS ou LEFT_ROADS
	// ======================================================
	int GetType(int i);

	// ======================
	// Dessine l'intersection
	// ======================
	void Draw(CFrustum * frustum, int elapsedTime);

private:

	// Pointeur sur la zone de l'intersection
	CZone *m_zone;

	// Identifiant de l'intersection
	id_t m_id;

	// Tableau des routes arrivant sur l'intersection
	// route = (m_id, m_roads[i]) avec 0 <= i <= 3
	// Il ne peut y avoir que 4 routes arrivant sur une intersection
	// Exemple: m_id = 4
	// m_roads[0] = 2, m_roads[1] = 8, m_roads[2] = 1, m_roads[3] = 6
	//
	//					1
	//					|
	//					|
	//			 6-----	4 -----8
	//					|
	//					|
	//					2
	//
	// la route m_roads[i] a toujours:
	//		- a sa droite: m_roads[(i+1)%4]
	//		- a sa gauche: m_roads[(i+3)%4]
	//		- devant lui : m_roads[(i+2)%4]
	//
	// Si m_roads[i] = -1 avec 0 <= i <= 3, la route n'existe pas
	int m_roads[4];

	// Type de l'intersection pour chaque route
	CObject *m_types[4];
};

#endif

