/*
 * 05/28/2003
 *
 * object.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "object.h"

// ##########################
// Classe CObject (abstraite)
// ##########################


// ================================
// Constructeur (identifiant + nom)
// ================================
CObject::CObject(id_t id, char *name) 
{ 
	m_id = id; 
	m_name = new char[MAX_NAME]; 
	strcpy(m_name,name); 
	m_type = OBJECT;
	m_vPosition	= CVector(0.0, 0.0, 0.0);
}


// ================
// Constructeur
// id = 0, nom = ""
// ================
CObject::CObject() 
{ 
	m_id = 0; 
	m_name = new char[MAX_NAME]; 
	strcpy(m_name,""); 
	m_type = OBJECT;
	m_vPosition	= CVector(0.0, 0.0, 0.0);
}


// ===========
// Destructeur
// ===========
CObject::~CObject()
{ 
	delete m_name;
}


// =============
// Change le nom
// =============
void CObject::SetName(char * name) 
{ 
	strcpy(m_name,name); 
}


// ===============
// Retourne le nom
// ===============
char * CObject::GetName() 
{ 
	return m_name; 
}


// ====================
// Change l'identifiant
// ====================
void CObject::SetID(id_t id) 
{ 
	m_id = id; 
}


// ======================
// Retourne l'identifiant
// =====================
id_t CObject::GetID() 
{
	return m_id; 
}


// ==============
// Change le type
// ==============
void CObject::SetType(int type) 
{ 
	m_type = type; 
}


// ================
// Retourne le type
// ================
int CObject::GetType() 
{ 
	return m_type; 
}


// ====================
// Retourne la position
// ====================
CVector CObject::GetPosition()	
{ 
	return m_vPosition;	
}


// ==================
// Change la position
// ==================
void CObject::SetPosition(float x, float y, float z)
{
	m_vPosition.x = x;
	m_vPosition.y = y;
	m_vPosition.z = z;
}


// ===============
// Dessine l'objet
// ===============
void CObject::Draw(CFrustum * frustum)
{
	// on dessine un repere a la position de l'objet
	glPushMatrix();
			
			glTranslatef(m_vPosition.x, m_vPosition.y, m_vPosition.z);

			glBegin(GL_LINES);
				glColor3f(1.0, 0.0, 0.0);
				glVertex3f(1.0, 0.0, 0.0);
				glVertex3f(0.0, 0.0, 0.0);
			
				glColor3f(0.0, 1.0, 0.0);
				glVertex3f(0.0, 1.0, 0.0);
				glVertex3f(0.0, 0.0, 0.0);

				glColor3f(0.0, 0.0, 1.0);
				glVertex3f(0.0, 0.0, 1.0);
				glVertex3f(0.0, 0.0, 0.0);
			glEnd();

			glColor3f(1.0, 0.0, 0.0);
			glTranslatef(1.0, 0.0, 0.0);
			glPushMatrix();
			glScalef(0.0015f, 0.0015f, 1.0f);
			glutStrokeCharacter(GLUT_STROKE_MONO_ROMAN, 'x');
			glPopMatrix();
			
			glColor3f(0.0, 1.0, 0.0);
			glTranslatef(-1.0, 1.0, 0.0);
			glPushMatrix();
			glScalef(0.0015f, 0.0015f, 1.0f);
			glutStrokeCharacter(GLUT_STROKE_MONO_ROMAN, 'y');
			glPopMatrix();

			glColor3f(0.0, 0.0, 1.0);
			glTranslatef(0.0, -1.0, 1.0);
			glPushMatrix();
			glScalef(0.0015f, 0.0015f, 1.0f);
			glutStrokeCharacter(GLUT_STROKE_MONO_ROMAN, 'z');
			glPopMatrix();
					
	glPopMatrix();
}
