/*
 * 05/28/2003
 *
 * road.h
 * Reprsente une route
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _ROAD_H
#define _ROAD_H

#include "global.h"


// ############
// Classe CRoad
// ############
class CRoad
{
public:
	
	// ============
	// Constructeur
	// ============
	CRoad(int inter1, int inter2, int direction);

	// ====================================
	// Change les informations sur la route
	// ====================================
	void SetInfoRoad(int inter1, int inter2, int direction);

	// ======================================
	// Retourne les informations sur la route
	// ======================================
	int GetInfoRoad(int i);

	// =======================================================
	// Retourne la capacit d'une route
	// (nombre de voiture quelle peut accueillir dans un sens)
	// =======================================================
	int GetCapacity();

	// =======================================================
	// Change la capacit d'une route
	// (nombre de voiture quelle peut accueillir dans un sens)
	// =======================================================
	void SetCapacity(int capacity);

	// =======================================
	// Change la taille d'un flot des voitures
	// =======================================
	void SetStream(int sens, float stream);

	// =========================================
	// Retourne la taille d'un flot des voitures
	// =========================================
	float GetStream(int sens);

	// =========================================
	// Ajoute a la taille d'un flot des voitures
	// =========================================
	void AddStream(int sens, float stream);

	// ============================================
	// Retranche a la taille d'un flot des voitures
	// ============================================
	void RemoveStream(int sens, float stream);

	// ================================
	// Teste si on peut ajouter un flot
	// ================================
	bool IsPossibleAddStream(int sens, float stream);

private:

	// Informations sur la route
	// [0] et [1] intersections, [2] sens X ou Y 
	int m_road[3];

	// Capacit d'une route
	// (nombre de voiture quelle peut accueillir dans un sens)
	int m_capacity;

	// Taille des flots de voiture dans chacun des 2 sens
	float m_stream[2];
};

#endif

