/*
 * 05/28/2003
 *
 * simtraffic.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "simtraffic.h"

char g_mapfile[50];
char g_graphicfile[50];
bool g_pause = false;
int g_time = 0;

bool g_showInfos = false;
bool g_showHelp = false;
bool g_showFps = false;

CMouse *g_mouse;
CTimerGLUT *g_timer;
CCameraSystem *g_cameras;
CBitmapGL *g_bitmap;
CTga *g_img;
CFrustum *g_frustum;
CCity *g_city;
CSkyBox *g_skybox;
CVehicle *g_vehicles = NULL;

// ====
// Main
// ====
int main(int argc, char ** argv)
{	
	glutInit(&argc, argv);
	initErrorLog();	
	initGLUT();
	initOpenGL();

	printf("\n%s\nrelease %s %s\n\n", APP_NAME, RELEASE, BUILD);
	printf("\nType H for help !\n");

	strcpy(g_mapfile, MAPS_DIR);
	strcpy(g_graphicfile, MAPS_DIR);
	if (argc == 2) {
		strcat(g_mapfile, argv[1]);
		strcat(g_graphicfile, argv[1]);
	}
	else {
		// Choix de  la ville
		switch(menuVille())
		{
		case 1:
			strcat(g_mapfile, "small");
			strcat(g_graphicfile, "small");
			break;
		case 2:
			strcat(g_mapfile, "medium");
			strcat(g_graphicfile, "medium");
			break;
		case 3:
			strcat(g_mapfile, "big");
			strcat(g_graphicfile, "big");
			break;
		}	
	}
	strcat(g_mapfile, ".city");
	strcat(g_graphicfile, ".gcity");

	init();
	glutMainLoop();
	return 0;
}

// ========================
// Initialisations diverses
// ========================
void init(void)
{
	// Initialise la fonction rand()
	srand(time(NULL));

	int error;

	// Allocation des variables globales
	g_mouse = new CMouse();
	g_timer = new CTimerGLUT();
	g_cameras = new CCameraSystem();
	g_bitmap = new CBitmapGL();
	g_frustum = new CFrustum();
	g_city = new CCity();
	if (g_city->LoadCity(g_mapfile) == false) { printf("Error see <log.txt> file\n"); exit(0); }
	g_city->LoadGraphics(g_graphicfile);
	g_skybox = new CSkyBox(POSZONE(400), POSZONE(200), POSZONE(400), 0, POSZONE(50), 0);
	g_img = new CTga();
	if ((error = g_img->Load("gfx/opengl.tga")) != TGA_OK)
		log("Unable to load \"gfx/opengl.tga\" image : error %d\n", error);

	// Chargement des textures de la skybox
	CTga texture = CTga();
	if ((error = texture.Load("gfx/sbox_bottom.tga")) != TGA_OK)
		log("Unable to load \"gfx/sbox_bottom.tga\" image : error %d\n", error);
	else g_skybox->InitTexture(SBOX_BOTTOM, &texture);
	if ((error = texture.Load("gfx/sbox_right.tga")) != TGA_OK)
		log("Unable to load \"gfx/sbox_right.tga\" image : error %d\n", error);
	else g_skybox->InitTexture(SBOX_RIGHT, &texture);
	if ((error = texture.Load("gfx/sbox_left.tga")) != TGA_OK)
		log("Unable to load \"gfx/sbox_left.tga\" image : error %d\n", error);
	else g_skybox->InitTexture(SBOX_LEFT, &texture);
	if ((error = texture.Load("gfx/sbox_top.tga")) != TGA_OK)
		log("Unable to load \"gfx/sbox_top.tga\" image : error %d\n", error);
	else g_skybox->InitTexture(SBOX_TOP, &texture);
	if ((error = texture.Load("gfx/sbox_front.tga")) != TGA_OK) 
		log("Unable to load \"gfx/sbox_front.tga\" image : error %d\n", error);
	else g_skybox->InitTexture(SBOX_FRONT, &texture);
	if ((error = texture.Load("gfx/sbox_back.tga")) != TGA_OK)
		log("Unable to load \"gfx/sbox_back.tga\" image : error %d\n", error);
	else g_skybox->InitTexture(SBOX_BACK, &texture);
	
	g_skybox->CreateDisplayLists();

	addVehicle();

	g_cameras->Add(new CCamera3Person(7,"Third person",g_vehicles));
	g_cameras->GetCurrent()->PositionCamera(0, 0, 0,   0, 0, 0,   0, 1, 0 );

	int w = g_city->GetWidth() - g_city->GetWidth()/3;
	int h = g_city->GetHeight() - g_city->GetHeight()/3;
	g_cameras->Add(new CCameraIso(5,"Iso"));
	g_cameras->GetCurrent()->PositionCamera(0, POSZONE(5), POSZONE(3),   0, 0, 0,   0, 1, 0 );

	g_cameras->Add(new CCamera(1,"Fixed 1"));
	g_cameras->GetCurrent()->PositionCamera(-POSZONE(w), POSZONE(h/2), +POSZONE(h),   0, POSZONE(1), 0,   0, 1, 0 );  

	g_cameras->Add(new CCamera(2,"Fixed 2"));
	g_cameras->GetCurrent()->PositionCamera(-POSZONE(w), POSZONE(h/2), -POSZONE(h),   0, POSZONE(1), 0,   0, 1, 0 ); 
	
	g_cameras->Add(new CCamera(3,"Fixed 3"));
	g_cameras->GetCurrent()->PositionCamera(POSZONE(w), POSZONE(h/2), -POSZONE(h),   0, POSZONE(1), 0,   0, 1, 0 ); 
	
	g_cameras->Add(new CCamera(6,"Fixed 4"));
	g_cameras->GetCurrent()->PositionCamera(POSZONE(w), POSZONE(h/2), POSZONE(h),   0, POSZONE(1), 0,   0, 1, 0 );  

	g_cameras->Add(new CCameraFree(4,"Free"));
	g_cameras->GetCurrent()->PositionCamera(0, POSZONE(2), POSZONE(5),   0, POSZONE(2), 0,   0, 1, 0 );
}
		
// ================================
// Fonction d'affichage de la scene
// ================================
void glutDisplay(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	
	g_cameras->GetCurrent()->Look();

	g_frustum->CalculateFrustum();
	
	g_cameras->UpdateAll();
	if (g_cameras->GetCurrent()->GetType() == CAMERA_3PERSON) 
		((CCamera3Person*)g_cameras->GetCurrent())->TestTarget(g_vehicles);

	#if DEBUG_MODE > 0
	g_cameras->DrawAll(NULL);
	#endif

	g_city->Draw(g_frustum, g_timer->FrameInterval());

	// Affichage des vehicules
	CNode *node = g_vehicles;
	((CVehicle *)node)->Draw(g_frustum);
	if (!g_pause) ((CVehicle *)node)->Animate(g_city, g_timer);
	for (node = node->Next() ; node != g_vehicles ; node = node->Next()) {
		((CVehicle *)node)->Draw(g_frustum);
		if (!g_pause) ((CVehicle *)node)->Animate(g_city, g_timer);
	}

	g_skybox->Draw();

	g_time += g_timer->FrameInterval();

	g_bitmap->Begin2D();
	if (g_showFps) g_bitmap->DrawString(glutGet(GLUT_WINDOW_WIDTH) - 67,glutGet(GLUT_WINDOW_HEIGHT) - 15,GLUT_BITMAP_8_BY_13,"%d fps", g_timer->GetFPS());
	
	if (g_showInfos) {
		g_bitmap->DrawString(5,glutGet(GLUT_WINDOW_HEIGHT) - 15, GLUT_BITMAP_8_BY_13,"Informations");
		g_bitmap->DrawString(5,glutGet(GLUT_WINDOW_HEIGHT) - 20, GLUT_BITMAP_8_BY_13,"_____________");
		g_bitmap->DrawString(5,glutGet(GLUT_WINDOW_HEIGHT) - 40, GLUT_BITMAP_8_BY_13,"City : %s",g_city->GetName());
		g_bitmap->DrawString(5,glutGet(GLUT_WINDOW_HEIGHT) - 55, GLUT_BITMAP_8_BY_13,"Display buildings : %d/%d",CBuilding::NB_DISPLAY,CBuilding::NB);
		g_bitmap->DrawString(5,glutGet(GLUT_WINDOW_HEIGHT) - 70, GLUT_BITMAP_8_BY_13,"Display vehicles : %d/%d",CVehicle::NB_DISPLAY,CVehicle::NB);
		g_bitmap->DrawString(5,glutGet(GLUT_WINDOW_HEIGHT) - 85, GLUT_BITMAP_8_BY_13,"Time : %d",g_time/1000);
	}
	if (g_showHelp) {
		g_bitmap->DrawString(5,glutGet(GLUT_WINDOW_HEIGHT) - 15, GLUT_BITMAP_8_BY_13,"%s v%s %s", APP_NAME, RELEASE, BUILD);
		g_bitmap->DrawString(5,glutGet(GLUT_WINDOW_HEIGHT) - 20, GLUT_BITMAP_8_BY_13,"________________________");

		g_bitmap->DrawString(5,glutGet(GLUT_WINDOW_HEIGHT) - 40, GLUT_BITMAP_8_BY_13,"i = show/hide informations");
		g_bitmap->DrawString(5,glutGet(GLUT_WINDOW_HEIGHT) - 55, GLUT_BITMAP_8_BY_13,"p = stop vehicles");
		g_bitmap->DrawString(5,glutGet(GLUT_WINDOW_HEIGHT) - 70, GLUT_BITMAP_8_BY_13,"left,down,right,up,left mouse button");
		g_bitmap->DrawString(5,glutGet(GLUT_WINDOW_HEIGHT) - 85, GLUT_BITMAP_8_BY_13,"q,s,d,z = move current camera");
		g_bitmap->DrawString(5,glutGet(GLUT_WINDOW_HEIGHT) - 100, GLUT_BITMAP_8_BY_13,"c = change current camera");
		g_bitmap->DrawString(5,glutGet(GLUT_WINDOW_HEIGHT) - 115, GLUT_BITMAP_8_BY_13,"+ = add one vehicle");
		g_bitmap->DrawString(5,glutGet(GLUT_WINDOW_HEIGHT) - 130, GLUT_BITMAP_8_BY_13,"- = del one vehicle");
		g_bitmap->DrawString(5,glutGet(GLUT_WINDOW_HEIGHT) - 145, GLUT_BITMAP_8_BY_13,"t = show/hide black outlines");
		g_bitmap->DrawString(5,glutGet(GLUT_WINDOW_HEIGHT) - 160, GLUT_BITMAP_8_BY_13,"f = show/hide frames per second");
		g_bitmap->DrawString(5,glutGet(GLUT_WINDOW_HEIGHT) - 175, GLUT_BITMAP_8_BY_13,"m = fill or line mode");
		g_bitmap->DrawString(5,glutGet(GLUT_WINDOW_HEIGHT) - 190, GLUT_BITMAP_8_BY_13,"esc = exit");
		g_bitmap->DrawString(glutGet(GLUT_WINDOW_WIDTH)-260,56,GLUT_BITMAP_8_BY_13,"Maitrise Informatique 2002-2003");
		g_bitmap->DrawString(glutGet(GLUT_WINDOW_WIDTH)-260,44,GLUT_BITMAP_8_BY_13,"Projet TER (Etude et recherche)");
		g_bitmap->DrawString(glutGet(GLUT_WINDOW_WIDTH)-260,40,GLUT_BITMAP_8_BY_13,"_______________________________");
		g_bitmap->DrawString(glutGet(GLUT_WINDOW_WIDTH)-260,27,GLUT_BITMAP_8_BY_13,"Vouriot Thierry");
		g_bitmap->DrawString(glutGet(GLUT_WINDOW_WIDTH)-260,14,GLUT_BITMAP_8_BY_13,"http://www.yeri.fr.st");
		g_bitmap->DrawImage(glutGet(GLUT_WINDOW_WIDTH) - g_img->GetWidth() - 5,glutGet(GLUT_WINDOW_HEIGHT) - g_img->GetHeight() - 20,g_img);
	}

	g_bitmap->DrawString(5,20, GLUT_BITMAP_8_BY_13,"_________________");
	g_bitmap->DrawString(5,5, GLUT_BITMAP_8_BY_13,"Camera : %s",g_cameras->GetCurrent()->GetName());
	g_bitmap->End2D();

	glFlush();
	glutSwapBuffers();

	CBuilding::NB_DISPLAY = 0;
	CVehicle::NB_DISPLAY = 0;
}

// ===========================================================
// Fonction excut lorsque le programme n'a plus rien  faire
// utilisee pour les animations
// ===========================================================
void glutIdle(void)
{
	g_timer->LockFPS(30);
	
	glutPostRedisplay();
}

// ==================================
// Gestion du clavier (touches ascii)
// ==================================
void glutKeyboard(unsigned char key, int x, int y)
{
	static bool   antialias = false;    // antialiasing
  	static GLenum polyMode  = GL_FILL;  // polygone mode
	
	float speed = 1.0f + (float)g_timer->FrameInterval()/75.0f;

	switch(key) {
		case ESC:	// touche escape
			terminateApplication();
			break;
		case 'a': antialias = !antialias;
				// activation de l'antialiasing
          		if (antialias)
          		{
            			glEnable(GL_POINT_SMOOTH);
            			glEnable(GL_LINE_SMOOTH);
            			glEnable(GL_POLYGON_SMOOTH);
            			glEnable(GL_BLEND);
          		}
          		// arrt de l'antialiasing
          		else
          		{
            			glDisable(GL_POINT_SMOOTH);
            			glDisable(GL_LINE_SMOOTH);
            			glDisable(GL_POLYGON_SMOOTH);
            			glDisable(GL_BLEND);
          		}
				break;
		case 'm':
			// passage du mode filaire au mode plein ou inversement
			if (polyMode == GL_FILL) polyMode = GL_LINE; else polyMode = GL_FILL;
			glPolygonMode(GL_FRONT_AND_BACK, polyMode);
			break;
		case 't':
			CBuilding::CELSHADING = !CBuilding::CELSHADING;
			CVehicle::CELSHADING = !CVehicle::CELSHADING;
			CTrafficLight::CELSHADING = !CTrafficLight::CELSHADING;
			break;	
		case 'p':
			g_pause = !g_pause;
			break;
		case 'z':
			g_cameras->GetCurrent()->MoveCamera(speed);
			break;
		case 's':
			g_cameras->GetCurrent()->MoveCamera(-speed);
			break;
		case 'd':
			g_cameras->GetCurrent()->StrafeCamera(speed);
			break;
		case 'q':
			g_cameras->GetCurrent()->StrafeCamera(-speed);
			break;
		case 'c': 
			g_cameras->Next();
			break;
		case '+':
		case '=':
			addVehicle(); 
			break;
		case '-':
			delVehicle();
			break;
		case 'f':
			g_showFps = !g_showFps;
			break;
		case 'i':
			g_showInfos = !g_showInfos;
			g_showHelp = false;
			break;
		case 'h':
			g_showHelp = !g_showHelp;
			g_showInfos = false;
			break;
	}
}

// ======================================
// Gestion du clavier (touches non ascii)
// ======================================
void glutSpecialKeys(int key, int x, int y)
{
	float speed = 1.0f + (float)g_timer->FrameInterval()/75.0;

	g_cameras->GetCurrent()->SetViewByKeyboard(key,speed);
}

// =========================================
// Gestion de la souris (clicks des boutons)
// =========================================
void glutMouse(int button, int state, int x, int y)
{
	if (button == GLUT_LEFT_BUTTON && state == GLUT_DOWN)
		g_mouse->left_button = true;
		
	else if (button == GLUT_LEFT_BUTTON && state == GLUT_UP)
		g_mouse->left_button = false;
		
	else if (button == GLUT_RIGHT_BUTTON && state == GLUT_DOWN)
		g_mouse->right_button = true;
		
	else if (button == GLUT_RIGHT_BUTTON && state == GLUT_UP)
		g_mouse->right_button = false;
	
	// sauvegarde de la position lors de l'appui sur un bouton
	g_mouse->x = x;
	g_mouse->y = y;
}

// ================================================================
// Gestion des mouvements de la souris lorsqu'un bouton est enfonce
// ================================================================
void glutMotion(int x, int y)
{
	if (g_mouse->left_button) {
		
		g_cameras->GetCurrent()->SetViewByMouse(x,y,g_mouse->x,g_mouse->y,100.0f);

		g_mouse->x = x;
		g_mouse->y = y;
		glutPostRedisplay();
	}
}



// ==========================================================
// Ajoute un vehicule aleatoirement sur une route de la ville
// ==========================================================
void addVehicle()
{
	srand(g_time);

	// on recupere toutes les zones ROAD de la ville
	CZone *roads = new CZone[g_city->GetHeight() * g_city->GetWidth()];
	int nbRoads = 0;
	for (int i = 0; i < g_city->GetHeight() * g_city->GetWidth(); i++) 
	{
		if ((g_city->GetZone(i)).GetType() == ROAD)
		{
			roads[nbRoads] = g_city->GetZone(i);
			nbRoads++;
		}
	}

	if (CVehicle::NB < nbRoads)		// on limite le nombre de vehicules
	{
		// On prend aleatoirement une zone ROAD ou il n'y a pas de vehicules
		CZone zone;
		bool ok = false;
		CVehicle *vehicle;
		while (!ok)
		{
			zone = roads[rand() % nbRoads];
			ok = true;

			// Teste si un vehicule est deja sur cette zone
			if (g_vehicles != NULL) 
			{
				vehicle = g_vehicles;
				if (vehicle->GetZoneX() == zone.x && vehicle->GetZoneY() == zone.y) ok = false;
				for (vehicle = (CVehicle*)(vehicle->Next()) ; vehicle != g_vehicles && ok; vehicle = (CVehicle*)(vehicle->Next())) 
				{
					if (vehicle->GetZoneX() == zone.x && vehicle->GetZoneY() == zone.y) ok = false;	
				}
			}
		}

		vehicle = new CVehicle(0,"");
		vehicle->Initialize(zone.x, zone.y, zone.GetRoad()->GetInfoRoad(2), zone.GetRoad()->GetInfoRoad(0), zone.GetRoad()->GetInfoRoad(1), g_city);
		
		zone.GetRoad()->AddStream(SENS(zone.GetRoad()->GetInfoRoad(2)),CAR_STREAM);
		// Ajoute le nouveau vehicule
		if (g_vehicles != NULL) g_vehicles->Add(vehicle);
		else g_vehicles = vehicle;
	}

	delete roads;
}

// ==============================================
// Supprime un vehicule (le premier de la chaine)
// ==============================================
void delVehicle()
{
	if (g_vehicles != g_vehicles->Next())	// si il reste plus qu'une voiture
	{
		g_vehicles->Del(g_city);
	}
}


// ===================================
// Menu permettant de choisir la ville
// ===================================
int menuVille()
{
	int ch;

	cout << "\nLoad the city :";
	cout << "\n  [1] Small";
	cout << "\n  [2] Medium";
	cout << "\n  [3] Big";
	cout << "\n\nChoice : ? ";

	cin >> ch;

	while (ch<=0 || ch>3)
	{
		cout << "\nError, enter your choice : ? ";
		cin >> ch;
	}

	return ch;
}

