/*
 * 05/28/2003
 *
 * tga.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "tga.h"

// ###########
// Classe CTga
// ###########


// ============
// Constructeur
// ============
CTga::CTga() {
	m_data = NULL;
}


// ===========
// Destructeur
// ===========
CTga::~CTga()
{
	delete m_data;
}
	

// ===================================================================
// Fonction lisant un fichier TGA 8, 16, 24 ou 32 bits (non compresse)
// ===================================================================
int CTga::Load(char *filename) 
{
	FILE *filePtr;			// pointeur sur le fichier
	unsigned char ucharBad;		// poubelle pour les unsigned char
	short int sintBad;		// poubelle pour les short int
	int colorMode;			// 4 pour RGBA ou 3 pour RGB
	long imageSize;			// taille de l'image TGA
	long imageIdx;			// compteur
	unsigned char colorSwap;	// variable d'change
	unsigned short pixels = 0;	// un pixel d'une image 16 bits
	int r, g, b;			// composantes RGB
	
	// ouverture du fichier TGA
	if ((filePtr = fopen (filename, "rb")) == NULL) {
        return TGA_FILE_NOT_FOUND;
	}
	
	// lecture des 2 premiers champs dont on n'a pas besoin
	fread(&ucharBad, sizeof(unsigned char), 1, filePtr);
	fread(&ucharBad, sizeof(unsigned char), 1, filePtr);
	
	// lecture du type de l'image
	fread(&ucharBad, sizeof(unsigned char), 1, filePtr);
	if ((ucharBad != 2) && (ucharBad != 3)) {
		fclose(filePtr);
		return TGA_BAD_IMAGE_TYPE;
	}
	// lecture des 5 champs suivants dont on n'a pas besoin
	fread(&sintBad, sizeof(short int), 1, filePtr);
	fread(&sintBad, sizeof(short int), 1, filePtr);
	fread(&ucharBad, sizeof(unsigned char), 1, filePtr);
	fread(&sintBad, sizeof(short int), 1, filePtr);
	fread(&sintBad, sizeof(short int), 1, filePtr);
	
	// lecture des dimensions de l'image
	fread(&m_width, sizeof(short int), 1, filePtr);
	fread(&m_height, sizeof(short int), 1, filePtr);
	fread(&m_bitCount, sizeof(unsigned char), 1, filePtr);
	
	// lecture du dernier champ dont on n'a pas besoin
	fread(&ucharBad, sizeof(unsigned char), 1, filePtr);
	
	// on ne lit que les images 8, 16, 24 ou 32 bits
	if (m_bitCount != 32 && m_bitCount != 24 && m_bitCount != 8 && m_bitCount != 16) {
		fclose(filePtr);
		return TGA_BAD_BITS;
	}
	
	// image 8, 24 ou 32 bits
	if(m_bitCount == 24 || m_bitCount == 32 || m_bitCount == 8)
	{
		// colorMode -> 3 = BGR, 4 = BGRA
		colorMode = m_bitCount / 8;
		imageSize = m_width * m_height * colorMode;
	
		// allocation de la mmoire pour les donnes de l'image
		//m_data = (unsigned char*) malloc(sizeof(unsigned char) * imageSize);
		m_data = new unsigned char[imageSize];

		// lecture des donnes de l'image
		fread(m_data, sizeof(unsigned char), imageSize, filePtr);
		if (m_data == NULL) {
        		fclose(filePtr);
			return TGA_BAD_DATA;
		}
	
		// dans les images TGA les couleurs des pixels sont en mode BGR
		// transformation en mode RGB
		if (m_bitCount != 8)
		for (imageIdx=0; imageIdx < imageSize; imageIdx += colorMode)
		{
			colorSwap = m_data[imageIdx];
			m_data[imageIdx] = m_data[imageIdx + 2];
			m_data[imageIdx + 2] = colorSwap;
		}
	}
	else { // image 16 bits

		colorMode = 3;	// mode RGB
		imageSize = m_width * m_height * colorMode;
		
		// allocation de la mmoire pour les donnes de l'image
		//tgaFile->imageData = (unsigned char*) malloc(sizeof(unsigned char) * imageSize);
		m_data = new unsigned char[imageSize];
		
		// en 16 bits chaque pixel est contenu dans un "unsigned short"
		// boucle sur tous les pixels de l'image
		for (imageIdx = 0; imageIdx < m_width * m_height; imageIdx++)
		{
			// lecture du pixel courant
			fread(&pixels, sizeof(unsigned short), 1, filePtr);
				
			// pour convertir 16 bits en composantes RGB
			// chaque composante occupe 5 bits
			// ex: pixels = 0x61E8 = 0110000111101000
			// on utilise un masque 0x1F = 11111
			// 0110000111101000 & 11111 = 01000
			// on effectue un dcallage de 3 bits pour avoir un nombre sur 8 bits
			// 01000 << 3 = 01000000 = 0x40 = 64 --> composante bleu
			// on dcale ensuite de 5 bits pour arriver  la prochaine
			// composante et on ritre la mme opration
			// 0110000111101000 >> 5 = 0000001100001111
			// 0000001100001111 & 11111 = 01111
			// 01111 << 3 = 01111000 = 0x78 = 120 --> composante vert
			// ...etc...
			b = (pixels & 0x1f) << 3;
			g = ((pixels >> 5) & 0x1f) << 3;
			r = ((pixels >> 10) & 0x1f) << 3;
				
			// on assigne ensuite les composantes au tableau de donnes
			m_data[imageIdx * 3 + 0] = r;
			m_data[imageIdx * 3 + 1] = g;
			m_data[imageIdx * 3 + 2] = b;
		}
	}
	
	// fermeture du fichier
	fclose(filePtr);
	
	// Format de l'image RGB, RGBA ou niveau de gris
	switch (m_bitCount) {
		case 8: m_texFormat = GL_LUMINANCE; break;
		case 32: m_texFormat = GL_RGBA; break;
		default: m_texFormat = GL_RGB;
	}
	
	return TGA_OK;
}


// ========================================================================
// Change les valeurs alpha des couleurs r,g,b en fonction du parametre cmp
// Si cmp > 0 on change les couleurs superieures a rgb
// Si cmp < 0 on change les couleurs inferieures a rgb
// Si cmp == 0 on change les couleurs egales a rgb
// ========================================================================
void CTga::AlphaForColor(unsigned char r, unsigned char g, unsigned char b, unsigned char alpha, char cmp)
{
	if (m_bitCount == 32)
	for (int imageIdx=3; imageIdx < (m_height * m_width * 4); imageIdx += 4)
	{
		if (cmp < 0)
		if (m_data[imageIdx-1] < b && m_data[imageIdx-2] < g && m_data[imageIdx-3] < r)
			m_data[imageIdx] = alpha;
		else if (cmp > 0)
		if (m_data[imageIdx-1] > b && m_data[imageIdx-2] > g && m_data[imageIdx-3] > r)
			m_data[imageIdx] = alpha;
		else if (cmp == 0)
		if (m_data[imageIdx-1] == b && m_data[imageIdx-2] == g && m_data[imageIdx-3] == r)
			m_data[imageIdx] = alpha;
	}
}


// ======================================================
// Fonction crant une image TGA dans le fichier filename
// ======================================================
int CTga::WriteTGAFile(char *filename, short int width, short int height, unsigned char bpp, unsigned char* imageData)
{
	unsigned char byteSkip;		// utilis pour mettre  0 certains champs
	short int shortSkip;		// utilis pour mettre  0 certains champs
	unsigned char imageType;	// type de l'image
	int colorMode;				// color mode
	unsigned char colorSwap;	// utilis pour la conversion RGB - BGR
	int imageIdx;				// compteur
	unsigned char bitDepth;		// image 24 bits
	long imageSize;				// taille de l'image
	FILE *filePtr;				// pointeur sur le fichier
	
	// cration du fichier en mode criture binaire
	if (!(filePtr = fopen (filename, "wb"))) {
		return TGA_FILE_NOT_CREATE;
	}
	
	imageType = 2;	// RGB uncompressed
	bitDepth = bpp;	// bitdepth
	colorMode = bpp / 8;	// color mode
	
	byteSkip = 0;
	shortSkip = 0;
	
	// criture des 2 premiers champs  0
	fwrite(&byteSkip, sizeof(unsigned char), 1, filePtr);
	fwrite(&byteSkip, sizeof(unsigned char), 1, filePtr);
	
	// criture du type de l'image
	fwrite(&imageType, sizeof(unsigned char), 1, filePtr);
	
	// criture des 5 champs suivants  0
	fwrite(&shortSkip, sizeof(short int), 1, filePtr);
	fwrite(&shortSkip, sizeof(short int), 1, filePtr);
	fwrite(&byteSkip, sizeof(unsigned char), 1, filePtr);
	fwrite(&shortSkip, sizeof(short int), 1, filePtr);
	fwrite(&shortSkip, sizeof(short int), 1, filePtr);
	
	// criture des dimensions de l'image
	fwrite(&width, sizeof(short int), 1, filePtr);
	fwrite(&height, sizeof(short int), 1, filePtr);
	fwrite(&bitDepth, sizeof(unsigned char), 1, filePtr);
	
	// criture du dernier champ  0
	fwrite(&byteSkip, sizeof(unsigned char), 1, filePtr);
	
	// calcul de la taille de l'image
	imageSize = width * height * colorMode;
	
	// conversion des donnes du mode RGB vers le mode BGR
	for (imageIdx = 0; imageIdx < imageSize ; imageIdx += colorMode)
	{
		colorSwap = imageData[imageIdx];
		imageData[imageIdx] = imageData[imageIdx + 2];
		imageData[imageIdx + 2] = colorSwap;
	}
	
	// criture des donnes de l'image dans le fichier
	fwrite(imageData, sizeof(unsigned char), imageSize, filePtr);
	
	// fermeture du fichier
	fclose(filePtr);
	
	return TGA_OK;	
}


// ====================================================================
// Retourne le format OpenGL de l'image (GL_RGB, GL_RGBA, GL_LUMINANCE)
// ====================================================================
GLenum CTga::GetTextureFormat()
{
	return m_texFormat;
}


// ==================================================
// Retourne le nombre de pixels en largeur de l'image
// ==================================================
short int CTga::GetWidth()
{
	return m_width;
}


// ==================================================
// Retourne le nombre de pixels en hauteur de l'image
// ==================================================
short int CTga::GetHeight()
{
	return m_height;
}


// ===============================================================================
// Retourne le nombre de bits par pixel (RGB = 24, RGBA = 32, Niveaux de gris = 8)
// ===============================================================================
unsigned char CTga::GetBitCount()
{
	return m_bitCount;
}


// ===================================================
// Retourne le tableau contenant les pixels de l'image
// ===================================================
unsigned char * CTga::GetData()
{
	return m_data;
}


// ================================================================
// Fonction permettant de tester si un entier est une puisance de 2
// ================================================================
bool CTga::CheckSize(int x)
{
    if (x == 2	 || x == 4 || 
        x == 8	 || x == 16 || 
        x == 32  || x == 64 ||
        x == 128 || x == 256 || 
	x == 512 || x == 1024)
        return true;
    else return false;
}

// ===================================================
// Creation d'une texture a partir de l'image courante
// ===================================================
int CTga::CreateTexture(GLuint id, bool mipmapping)
{
	glBindTexture(GL_TEXTURE_2D, id);
		
	if (!mipmapping) {
		// filtrage plus rapide
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		
		glTexImage2D(GL_TEXTURE_2D, 0,  m_texFormat, m_width, m_height, 
			0,  m_texFormat, GL_UNSIGNED_BYTE, m_data);
	}
	else {
		// Mipmapping: meilleur qualit d'affichage mais plus lent
		gluBuild2DMipmaps(GL_TEXTURE_2D, m_texFormat, m_width, m_height, m_texFormat, GL_UNSIGNED_BYTE, m_data);

		// filtrage offrant une meilleur qualit	
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	}
	
	// l'option GL_CLAMP_TO_EDGE permet de ne pas rpter la texture comme GL_REPEAT
	// et vite donc des effets indsirables, de plus elle permet de ne pas voir
	// les bordures des textures contrairement  GL_CLAMP
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

	//delete m_data;

	return 1;
}


// log
// 30/08/2002 prise en charge des images 8 bits
// 09/09/2002 prise en charge des images 16 bits
// 23/02/2003 empaquetage des fonctions dans une classe CTga
 

// Le format Targa TGA est divis en 2 partie l'en-tte et les donnes.
// L'en-tte contient 12 champs et est dfinie par la structure suivante :
//
// typedef struct tagTARGAFILEHEADER
// {
//	unsigned char imageIDLength;	// number of characters in identification field;
//									// 0 denotes no identification field is included
//	unsigned char colorMapType;		// type of color map; always 0
//	unsigned char imageTypeCode;	// uncompressed RGB is 2;
//									// uncompressed grayscale is 3
//	short int colorMapOrigin;		// origin of color map (lo-hi); always 0
//	short int colorMapLength;		// length of color map (lo-hi); always 0
//	short int colorMapEntrySize;	// color map entry size (lo-hi); always 0
//	short int imageXOrigin;			// x coordinate of lower-left corner of image
//									// (lo-hi); always 0
//	short int imageYOrigin;			// y coordinate of lower-left corner of image
//									// (lo-hi); always 0
//	short int imageWidth;			// width of image in pixels (lo-hi)
//	short int imageHeight;			// height of image in pixels (lo-hi)
//	unsigned char bitCount;			// number of bits; 16, 24, 32
//	unsigned char imageDescriptor;	// 24 bit = 0x00; 32 bit = 0x08
// } TARGAFILEHEADER
 
