/*
 * 05/28/2003
 *
 * timerGLUT.h
 * Classe permettant de recuperer le temps ecoule et 
 * de calculer le nombre d'images par seconde
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef _TIMERGLUT_H
#define _TIMERGLUT_H

#include <GL/glut.h>

// #################
// Classe CTimerGLUT
// #################
class CTimerGLUT
{
public:

	// =======================
	// Constructeur par defaut
	// =======================
	CTimerGLUT();

	// ===========
	// Destructeur
	// ===========
	~CTimerGLUT();

	// ==============================================================
	// Retourne le temps en millisecondes entre 2 frames
	// Attention cette valeur n'est disponible que lorsque
	// GetFPS() ou LockFPS() ou GetElapsedSeconds() on ete utilise
	// mais on ne doit utiliser qu'une seule de ces 3 fonctions
	// entre l'affichage de 2 frames sinon celle-ci n'est plus valide
	// ==============================================================
	int FrameInterval();

	// ==============================================================
	// Retourne le temps en millisecondes depuis la creation du timer
	// ou depuis le dernier appel de GetElapsedSeconds()
	// ==============================================================
	int GetElapsedSeconds();

	// ==========================================================
	// Calcul le nombre de frames par secondes de l'affichage,
	// le parametres elapsedFrames correspond au nombre de frames
	// affichees depuis le dernier appel de GetFPS()
	// ==========================================================
	int CalcFPS(unsigned long elapsedFrames = 1);

	// =============================================================
	// Permet de limiter de nombre de frames par secondes a un
	// certain maximum, cette methode doit etre appelee entre chaque
	// frames
	// =============================================================
	int LockFPS(unsigned char targetFPS);

	// ======================================
	// Retourne le dernier frame rate calcule
	// ======================================
	int GetFPS();

private:

	// Temps au moment de l'initialisation de la classe
	int m_startTime;

	// Temps en millisecondes entre 2 frames
	int m_frameInterval;

	// Nombre de frames par seconde (courant)
	int m_fps;
 
};

#endif

