/*
 * 05/28/2003
 *
 * trafficlight.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "trafficlight.h"

// ####################
// Classe CTrafficLight
// ####################


// Dessin en cel-shading
bool CTrafficLight::CELSHADING = true;	


// ======================
// Constructeur
// light = couleur du feu
// ======================
CTrafficLight::CTrafficLight(int light) : CObject(0, "")
{
	m_type = TRAFFIC_LIGHT;
	m_light = light;

	switch (m_light)
	{
		case RED: m_time = RED_TIMEOUT; break;
		case GREEN: m_time = GREEN_TIMEOUT; break;
		case ORANGE: m_time = ORANGE_TIMEOUT; break;
	}
}


// ===========
// Destructeur
// ===========
CTrafficLight::~CTrafficLight()
{ }


// ==============
// Dessine le feu
// ==============
void CTrafficLight::Draw(CFrustum * frustum)
{
	#if DEBUG_MODE > 0
	CObject::Draw(frustum);
	#endif

	int polyMode[2];
	glGetIntegerv(GL_POLYGON_MODE, polyMode);

	glDisable(GL_CULL_FACE);
	
	glPushMatrix();
	glTranslatef(m_vPosition.x, m_vPosition.y, m_vPosition.z);

	glPushMatrix();
	glTranslatef(0.0, ZONE_SIZE/4.0 + ZONE_SIZE/1000.0f, 0.0);
	glColor3f(0.4, 0.4, 0.4);
	glScalef(ZONE_SIZE/25.0, ZONE_SIZE/2.0, ZONE_SIZE/25.0);
	glutSolidCube(1.0);
	glColor3f(0.0, 0.0, 0.0);
	if (CELSHADING && polyMode[0] == GL_FILL) glutWireCube(1.001);
	glPopMatrix();
	
	glPushMatrix();
	glTranslatef(0.0, ZONE_SIZE/2.0, 0.0);
	glColor3f(0.2, 0.2, 0.2);
	glScalef(ZONE_SIZE/15.0, ZONE_SIZE/4.0, ZONE_SIZE/15.0);
	glutSolidCube(1.0);
	glColor3f(0.0, 0.0, 0.0);
	if (CELSHADING && polyMode[0] == GL_FILL) glutWireCube(1.001);
	glPopMatrix();

	glPushMatrix();
	glTranslatef(ZONE_SIZE/20.0, 5.8*ZONE_SIZE/10.0, 0.0);
	if (m_light == RED) glColor3f(1.0, 0.0, 0.0);
	else glColor3f(0.4, 0.4, 0.4);
	glScalef(ZONE_SIZE/45.0, ZONE_SIZE/28.0, ZONE_SIZE/28.0);
	glutSolidCube(1.3);
	glColor3f(0.0, 0.0, 0.0);
	if (CELSHADING && polyMode[0] == GL_FILL) glutWireCube(1.301);
	glPopMatrix();

	glPushMatrix();
	glTranslatef(ZONE_SIZE/20.0, 5.1*ZONE_SIZE/10.0, 0.0);
	if (m_light == ORANGE) glColor3f(1.0, 0.5, 0.0);
	else glColor3f(0.4, 0.4, 0.4);
	glScalef(ZONE_SIZE/45.0, ZONE_SIZE/28.0, ZONE_SIZE/28.0);
	glutSolidCube(1.3);
	glColor3f(0.0, 0.0, 0.0);
	if (CELSHADING && polyMode[0] == GL_FILL) glutWireCube(1.301);
	glPopMatrix();

	glPushMatrix();
	glTranslatef(ZONE_SIZE/20.0, 4.4*ZONE_SIZE/10.0, 0.0);
	if (m_light == GREEN) glColor3f(0.0, 1.0, 0.0);
	else glColor3f(0.4, 0.4, 0.4);
	glScalef(ZONE_SIZE/45.0, ZONE_SIZE/28.0, ZONE_SIZE/28.0);
	glutSolidCube(1.3);
	glColor3f(0.0, 0.0, 0.0);
	if (CELSHADING && polyMode[0] == GL_FILL) glutWireCube(1.301);
	glPopMatrix();

	glPopMatrix();

	glEnable(GL_CULL_FACE);
}


// =================================
// Teste si le feux doit etre change
// =================================
void CTrafficLight::Change(int elapsedTime)
{
	m_time -= elapsedTime;
	if (m_time <= 0)
	{
		if (m_light == RED) 
		{
			m_light = GREEN;
			m_time = GREEN_TIMEOUT;
		}
		else if (m_light == GREEN) 
		{
			m_light = ORANGE;
			m_time = ORANGE_TIMEOUT;
		}
		else if (m_light == ORANGE) 
		{
			m_light = RED;
			m_time = RED_TIMEOUT;
		}	
	}
}


// ===========================================
// Retourne la couleur du feu RED GREEN ORANGE
// ===========================================
int CTrafficLight::GetLight()
{
	return m_light;
}
