/*
 * 05/28/2003
 *
 * vehicle.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "vehicle.h"

// ###############
// Classe CVehicle
// ###############


// Nombre de vehicules affiches
// Doit tre remis a zero apres chaque affichage de frame
int CVehicle::NB_DISPLAY = 0;


// Dessin en cel-shading
bool CVehicle::CELSHADING = true;	


// Nombre d'instances de CVehicule
int CVehicle::NB = 0;


// ============
// Constructeur
// ============
CVehicle::CVehicle(id_t id, char * name) : CObject(id, name), CNode()
{
	NB++;
	m_type = VEHICLE;

	float w = ZONE_SIZE / 4.0f;		// Largeur du vehicule en fonction de la taille d'une zone
	float h = ZONE_SIZE / 10.0f;	// Hauteur du vehicule en fonction de la taille d'une zone
	float l = CAR_LENGTH;			// Longueur du vehicule en fonction de la taille d'une zone

	float dec = (float)ZONE_SIZE / 40.0f;	

	// Definitions des points
	m_points[0][0] = -(w/2.0f);		m_points[0][1] = 0.05;		m_points[0][2] = -(l/2.0f);
	m_points[1][0] = +(w/2.0f);		m_points[1][1] = 0.05;		m_points[1][2] = -(l/2.0f);
	m_points[2][0] = +(w/2.0f);		m_points[2][1] = 0.05;		m_points[2][2] = +(l/2.0f);
	m_points[3][0] = -(w/2.0f);		m_points[3][1] = 0.05;		m_points[3][2] = +(l/2.0f);

	m_points[4][0] = -(w/2.0f);		m_points[4][1] = h;			m_points[4][2] = -(l/2.0f);
	m_points[5][0] = +(w/2.0f);		m_points[5][1] = h;			m_points[5][2] = -(l/2.0f);
	m_points[6][0] = +(w/2.0f);		m_points[6][1] = h;			m_points[6][2] = +(l/2.0f);
	m_points[7][0] = -(w/2.0f);		m_points[7][1] = h;			m_points[7][2] = +(l/2.0f);

	m_points[8][0] = -(w/2.0f);		m_points[8][1] = h;			m_points[8][2] = -(l/1.5f)/4.0f;
	m_points[9][0] = +(w/2.0f);		m_points[9][1] = h;			m_points[9][2] = -(l/1.5f)/4.0f;
	m_points[10][0] = +(w/2.0f);	m_points[10][1] = h;		m_points[10][2] = 3.5f*(l/2.5f)/4.0f;
	m_points[11][0] = -(w/2.0f);	m_points[11][1] = h;		m_points[11][2] = 3.5f*(l/2.5f)/4.0f;

	m_points[12][0] = -(w/2.0f);	m_points[12][1] = h + h/2;		m_points[12][2] = -(l/1.5f)/4.0f;
	m_points[13][0] = +(w/2.0f);	m_points[13][1] = h + h/2;		m_points[13][2] = -(l/1.5f)/4.0f;
	m_points[14][0] = +(w/2.0f);	m_points[14][1] = h + h/2;		m_points[14][2] = 3.5f*(l/2.5f)/4.0f;
	m_points[15][0] = -(w/2.0f);	m_points[15][1] = h + h/2;		m_points[15][2] = 3.5f*(l/2.5f)/4.0f;

	CreateDisplayLists();

	m_tabRot[X] = 270;
	m_tabRot[Y] = 180;
	m_tabRot[MX] = 90;
	m_tabRot[MY] = 0;

	m_speed = 0.1f;
	m_stop = false;
	m_stopVehicle = false;
	m_stopInter = false;

	m_width = w;
	m_length = l;

	m_inInter = -1;

	m_stopRightPriority = false;
}


// ===========
// Destructeur
// ===========
CVehicle::~CVehicle() 
{ 
	if (glIsList(m_displayList[0])) glDeleteLists(m_displayList[0], 1);
	if (glIsList(m_displayList[1])) glDeleteLists(m_displayList[1], 1);
	NB--;m_zoneX = -1;
}


// ===========================================
// Creation des listes d'affichage du vehicule
// ===========================================
void CVehicle::CreateDisplayLists()
{
	if (glIsList(m_displayList[0])) glDeleteLists(m_displayList[0], 1);
	m_displayList[0] = glGenLists(1);

	glNewList(m_displayList[0], GL_COMPILE);
		// Affichage du parrallepipede
		glBegin(GL_QUADS);
			
			glColor3f(0.96, 0.57, 0.1);
			
			// BOTTOM
			glNormal3f(0.0, -1.0, 0.0);
			glVertex3fv(m_points[0]);
			glVertex3fv(m_points[1]);
			glVertex3fv(m_points[2]);
			glVertex3fv(m_points[3]);
			// TOP
			glNormal3f(0.0, 1.0, 0.0);
			glVertex3fv(m_points[4]);
			glVertex3fv(m_points[5]);
			glVertex3fv(m_points[6]);
			glVertex3fv(m_points[7]);
			// FRONT
			glNormal3f(0.0, 0.0, 1.0);
			glVertex3fv(m_points[0]);
			glVertex3fv(m_points[1]);
			glVertex3fv(m_points[5]);
			glVertex3fv(m_points[4]);
			// RIGHT
			glNormal3f(1.0, 0.0, 0.0);
			glVertex3fv(m_points[1]);
			glVertex3fv(m_points[2]);
			glVertex3fv(m_points[6]);
			glVertex3fv(m_points[5]);
			// BACK
			glNormal3f(0.0, 0.0, -1.0);
			glVertex3fv(m_points[2]);
			glVertex3fv(m_points[3]);
			glVertex3fv(m_points[7]);
			glVertex3fv(m_points[6]);
			// LEFT
			glNormal3f(-1.0, 0.0, 0.0);
			glVertex3fv(m_points[3]);
			glVertex3fv(m_points[0]);
			glVertex3fv(m_points[4]);
			glVertex3fv(m_points[7]);

		
			// TOP
			glNormal3f(0.0, 1.0, 0.0);
			glVertex3fv(m_points[12]);
			glVertex3fv(m_points[13]);
			glVertex3fv(m_points[14]);
			glVertex3fv(m_points[15]);
			// FRONT
			glNormal3f(0.0, 0.0, 1.0);
			glVertex3fv(m_points[8]);
			glVertex3fv(m_points[9]);
			glVertex3fv(m_points[13]);
			glVertex3fv(m_points[12]);
			// RIGHT
			glNormal3f(1.0, 0.0, 0.0);
			glVertex3fv(m_points[9]);
			glVertex3fv(m_points[10]);
			glVertex3fv(m_points[14]);
			glVertex3fv(m_points[13]);
			// BACK
			glNormal3f(0.0, 0.0, -1.0);
			glVertex3fv(m_points[10]);
			glVertex3fv(m_points[11]);
			glVertex3fv(m_points[15]);
			glVertex3fv(m_points[14]);
			// LEFT
			glNormal3f(-1.0, 0.0, 0.0);
			glVertex3fv(m_points[11]);
			glVertex3fv(m_points[8]);
			glVertex3fv(m_points[12]);
			glVertex3fv(m_points[15]);

		glEnd();

	glEndList();

	if (glIsList(m_displayList[1])) glDeleteLists(m_displayList[1], 1);
	m_displayList[1] = glGenLists(1);
	float d = 0.005;

	glNewList(m_displayList[1], GL_COMPILE);

		// Affichage des traits noirs pour simuler le cell-shading
		glLineWidth(1.0);
		glBegin(GL_LINES);
		
			glColor3f(0.0,0.0,0.0);

			glVertex3f(m_points[0][0]-d, m_points[0][1]+d, m_points[0][2]-d);
			glVertex3f(m_points[1][0]+d, m_points[1][1]+d, m_points[1][2]-d);
			glVertex3f(m_points[1][0]+d, m_points[1][1]+d, m_points[1][2]-d);
			glVertex3f(m_points[2][0]+d, m_points[2][1]+d, m_points[2][2]+d);
			glVertex3f(m_points[2][0]+d, m_points[2][1]+d, m_points[2][2]+d);
			glVertex3f(m_points[3][0]-d, m_points[3][1]+d, m_points[3][2]+d);
			glVertex3f(m_points[3][0]-d, m_points[3][1]+d, m_points[3][2]+d);
			glVertex3f(m_points[0][0]-d, m_points[0][1]+d, m_points[0][2]-d);

			glVertex3f(m_points[4][0]-d, m_points[4][1], m_points[4][2]-d);
			glVertex3f(m_points[5][0]+d, m_points[5][1], m_points[5][2]-d);
			glVertex3f(m_points[5][0]+d, m_points[5][1], m_points[5][2]-d);
			glVertex3f(m_points[6][0]+d, m_points[6][1], m_points[6][2]+d);
			glVertex3f(m_points[6][0]+d, m_points[6][1], m_points[6][2]+d);
			glVertex3f(m_points[7][0]-d, m_points[7][1], m_points[7][2]+d);
			glVertex3f(m_points[7][0]-d, m_points[7][1], m_points[7][2]+d);
			glVertex3f(m_points[4][0]-d, m_points[4][1], m_points[4][2]-d);

			glVertex3f(m_points[1][0]+d, m_points[1][1]+d, m_points[1][2]-d);
			glVertex3f(m_points[5][0]+d, m_points[5][1], m_points[5][2]-d);

			glVertex3f(m_points[2][0]+d, m_points[2][1]+d, m_points[2][2]+d);
			glVertex3f(m_points[6][0]+d, m_points[6][1], m_points[6][2]+d);

			glVertex3f(m_points[3][0]-d, m_points[3][1]+d, m_points[3][2]+d);
			glVertex3f(m_points[7][0]-d, m_points[7][1], m_points[7][2]+d);

			glVertex3f(m_points[0][0]-d, m_points[0][1]+d, m_points[0][2]-d);
			glVertex3f(m_points[4][0]-d, m_points[4][1], m_points[4][2]-d);

			glVertex3f(m_points[8][0]-d, m_points[8][1], m_points[8][2]-d);
			glVertex3f(m_points[9][0]+d, m_points[9][1], m_points[9][2]-d);
			glVertex3f(m_points[9][0]+d, m_points[9][1], m_points[9][2]-d);
			glVertex3f(m_points[10][0]+d, m_points[10][1], m_points[10][2]+d);
			glVertex3f(m_points[10][0]+d, m_points[10][1], m_points[10][2]+d);
			glVertex3f(m_points[11][0]-d, m_points[11][1], m_points[11][2]+d);
			glVertex3f(m_points[11][0]-d, m_points[11][1], m_points[11][2]+d);
			glVertex3f(m_points[8][0]-d, m_points[8][1], m_points[8][2]-d);

			glVertex3f(m_points[12][0]-d, m_points[12][1], m_points[12][2]-d);
			glVertex3f(m_points[13][0]+d, m_points[13][1], m_points[13][2]-d);
			glVertex3f(m_points[13][0]+d, m_points[13][1], m_points[13][2]-d);
			glVertex3f(m_points[14][0]+d, m_points[14][1], m_points[14][2]+d);
			glVertex3f(m_points[14][0]+d, m_points[14][1], m_points[14][2]+d);
			glVertex3f(m_points[15][0]-d, m_points[15][1], m_points[15][2]+d);
			glVertex3f(m_points[15][0]-d, m_points[15][1], m_points[15][2]+d);
			glVertex3f(m_points[12][0]-d, m_points[12][1], m_points[12][2]-d);

			glVertex3f(m_points[8][0]+d, m_points[8][1]+d, m_points[8][2]-d);
			glVertex3f(m_points[12][0]+d, m_points[12][1], m_points[12][2]-d);

			glVertex3f(m_points[9][0]+d, m_points[9][1]+d, m_points[9][2]+d);
			glVertex3f(m_points[13][0]+d, m_points[13][1], m_points[13][2]+d);

			glVertex3f(m_points[10][0]-d, m_points[10][1]+d, m_points[10][2]+d);
			glVertex3f(m_points[14][0]-d, m_points[14][1], m_points[14][2]+d);

			glVertex3f(m_points[11][0]-d, m_points[11][1]+d, m_points[11][2]-d);
			glVertex3f(m_points[15][0]-d, m_points[15][1], m_points[15][2]-d);

		glEnd();
		glLineWidth(1.0);
	glEndList();	
}


// ===================
// Dessine le vehicule
// ===================
void CVehicle::Draw(CFrustum * frustum)
{
	if (frustum == NULL || frustum->CubeInFrustum(m_vPosition.x, m_vPosition.y, m_vPosition.z, m_length/2.0f))
	{
		#if DEBUG_MODE > 0
		CObject::Draw(frustum);
		#endif
		NB_DISPLAY++;

		int polyMode[2];
		glGetIntegerv(GL_POLYGON_MODE, polyMode);
		glPushMatrix();
			
			glTranslatef(m_vPosition.x, m_vPosition.y, m_vPosition.z);
			glRotatef(m_rotation, 0.0, 1.0, 0.0);
			
			glCallList(m_displayList[0]);
			if (CELSHADING && polyMode[0] == GL_FILL) glCallList(m_displayList[1]);
			
		glPopMatrix();
	}
}


// ===============================================================================
// Place et oriente le vehicule sur la ville a la position x,y
// direction : direction logique X, Y, MX, MY
// inter1, inter2 : route sur laquelle circule le vehicule (de inter1 vers inter2)
// city : ville
// ===============================================================================
void CVehicle::Initialize(unit_t x, unit_t y, int direction, int inter1, int inter2, CCity * city)
{
	SetPosition(city->GetPosition().x + POSZONE(x), city->GetPosition().y, city->GetPosition().z + POSZONE(y));
	
	m_direction = direction;
	m_zoneX = x;
	m_zoneY = y;
	m_inter1 = inter1;
	m_inter2 = inter2;

	switch (m_direction)
	{
		case X :	m_vPosition.z += 3*ZONE_SIZE/4.0f;
					m_vPosition.x += ZONE_SIZE/2.0f;
					break;
		case Y :	m_vPosition.x += ZONE_SIZE/4.0f;
					m_vPosition.z += ZONE_SIZE/2.0f;
					break;
		case MX :	m_vPosition.z += ZONE_SIZE/4.0f;
					m_vPosition.x += ZONE_SIZE/2.0f;
					break;
		case MY :	m_vPosition.x += 3*ZONE_SIZE/4.0f;
					m_vPosition.z += ZONE_SIZE/2.0f;
					break;
	}
	
	m_rotation = m_tabRot[m_direction];

	SetRandomNextTurn(city);
}


// ==========================================================
// Recupere la coordonnee de la zone precedente d'une voiture
// ==========================================================
unit_t CVehicle::Prevzone(int c)
{
	if (c == X)
	{
		if (m_direction == X) return m_zoneX-1;
		if (m_direction == MX) return m_zoneX+1;
		return m_zoneX;
	}
	else
	{
		if (m_direction == Y) return m_zoneY-1;
		if (m_direction == MY) return m_zoneY+1;
		return m_zoneY;
	}
}


// =====================================================================
// Retourne la prochaine position X ou Y (c) en fonction de la direction
// et de la precedente position
// =====================================================================
static inline unit_t nextZone(int c, int direction, int zone)
{
	if (c == X)
	{
		if (direction == X) return zone+1;
		if (direction == MX) return zone-1;
		else return zone;
	}
	else
	{
		if (direction == Y) return zone+1;
		if (direction == MY) return zone-1;
		else return zone;
	}
}


// ========================
// Fait evoluer le vehicule 
// ========================
void CVehicle::Animate(CCity * city, CTimerGLUT *timer)
{
	if (!m_stop ) MoveForward(/*((float)timer->FrameInterval()/200.0f) * */m_speed * ZONE_SIZE/2, city->GetPosition());
	
	CIntersection *inter = city->GetGraph()->GetInter(m_inter2);

	m_stopVehicle = VehiculeFront();

	// On est en train de franchir une intersection
	if (m_inInter != -1 && !m_stop) 
	{
		Turn(city);
	}

	// On teste si on entre dans l'intersection d'arrivee de la route
	else if ( inter->GetZone()->x == m_zoneX && inter->GetZone()->y == m_zoneY && m_inInter == -1)
	{
		// On met  jour le flot dans l'ancienne route	
		CZone zone = city->GetZone(Prevzone(X), Prevzone(Y));
		if (zone.GetType() == ROAD) zone.GetRoad()->RemoveStream(SENS(m_direction),CAR_STREAM);		

		// On met  jour le flot dans la nouvelle route	
		zone = city->GetZone(nextZone(X,m_nextDirection,inter->GetZone()->x),nextZone(Y,m_nextDirection,inter->GetZone()->y));
		if (zone.GetType() == ROAD) zone.GetRoad()->AddStream(SENS(m_nextDirection),CAR_STREAM);

		SetRoad(m_inter2, inter->GetRoad(m_nextDirection));

		m_inInter = m_nextTurn;	// on est dans l'intersection
	}

	// Si la prochaine zone ou l'on va rentrer est l'intersection d'arrivee de la route
	if (inter->GetZone()->x == m_nextZoneX  && inter->GetZone()->y == m_nextZoneY && m_inInter == -1)
	{
		// On regarde le type de la prochaine intersection
		int type = inter->GetType((m_direction+2)%4);
		CZone nextZ = city->GetZone(nextZone(X,m_nextDirection,inter->GetZone()->x),nextZone(Y,m_nextDirection,inter->GetZone()->y));
		if (type == RED || type == ORANGE)
		{
			m_nextInter = false;
		}
		else if (inter->GetType((m_direction+2)%4) == GREEN || inter->GetType((m_direction+2)%4) == TRANSP)
		{
			if (nextZ.GetType() == ROAD && nextZ.GetRoad()->IsPossibleAddStream(SENS(m_nextDirection),CAR_STREAM)) {
				m_nextInter = true;
				m_stopInter = false;
			}
			else if (m_inInter == -1 && nextZ.GetType() == ROAD) {
				m_nextInter = false;
			}
		}
	
		// on arrete le vehicule avant d'arriver a l'intersection (non franchissable)
		float posVeh, posZone;
		if (m_nextInter == false)
		switch(m_direction)
		{
			case X:
				posVeh = m_vPosition.x - city->GetPosition().x + m_length*0.5f;
				posZone = POSZONE(m_nextZoneX);
				if (posVeh >= posZone) m_stopInter = true;
				break;
			case MX:
				posVeh = m_vPosition.x - city->GetPosition().x - m_length*0.5f;
				posZone = POSZONE(m_nextZoneX) + ZONE_SIZE;
				if (posVeh <= posZone) m_stopInter = true;
				break;
			case Y:
				posVeh = m_vPosition.z - city->GetPosition().z + m_length*0.5f;
				posZone = POSZONE(m_nextZoneY);
				if (posVeh >= posZone) m_stopInter = true;
				break;
			case MY:
				posVeh = m_vPosition.z - city->GetPosition().z - m_length*0.5f;
				posZone = POSZONE(m_nextZoneY) + ZONE_SIZE;
				if (posVeh <= posZone) m_stopInter = true;
				break;
		}
		if (m_stopRightPriority && m_inInter == -1) m_stopInter = true;
	}

	if (m_stopInter || m_stopVehicle) m_stop = true;
	else m_stop = false;

	// dans une intersection on ne s'arrete pas
	if (m_inInter != -1) m_stop = false;
}


// ================================================
// Test si (i == k && j == l) ou (j == k && i == l)
// ================================================
static inline bool equal2p2(int i, int j, int k, int l)
{
	if (i == k && j == l) return true;
	if (j == k && i == l) return true;
	return false;
}


// ===============================
// Teste si il y a vehicule devant
// ===============================
bool CVehicle::VehiculeFront()
{
	m_stopRightPriority = false;

	// On teste tous les vehicules
	CVehicle *vehicle = this;
	for (vehicle = (CVehicle*)(vehicle->Next()) ; vehicle != this; vehicle = (CVehicle*)(vehicle->Next())) 
	{
		// Si le vehicule est sur la meme routes et dans la meme direction que le vehicule courant
		if (equal2p2(m_inter1, m_inter2, vehicle->m_inter1, vehicle->m_inter2) && m_direction == vehicle->m_direction && vehicle->m_inInter == -1)
		{
			float pos1, pos2;
			// On fait le test si le vehicule est arrete dans une intersection
			// ou si celui ci n'est pas dans une intersection
			if ((vehicle->m_inInter != -1 && vehicle->m_stop) || vehicle->m_inInter == -1)
			switch(m_direction)
			{
				case X:
					pos1 = m_vPosition.x;
					pos2 = vehicle->m_vPosition.x;
					if (pos1 < pos2) 
					if ((pos1 + m_length+SECURE_DIST) >= pos2) return true;
					break;
				case MX:
					pos1 = m_vPosition.x;
					pos2 = vehicle->m_vPosition.x;
					if (pos1 > pos2) 
					if ((pos2 + m_length+SECURE_DIST) >= pos1) return true;
					break;
				case Y:
					pos1 = m_vPosition.z;
					pos2 = vehicle->m_vPosition.z;
					if (pos1 < pos2)
					if ((pos1 + m_length+SECURE_DIST) >= pos2) return true;
					
					break;
				case MY:
					pos1 = m_vPosition.z;
					pos2 = vehicle->m_vPosition.z;
					if (pos1 > pos2)
					if ((pos2 + m_length+SECURE_DIST) >= pos1) return true;
					break;
			}
		}
		// Teste la priorit a droite
		if (m_nextTurn == LEFT_TURN && !m_stopRightPriority && m_inInter == -1)
		{
			// si il y a un vehicule en sens inverse ne tournant pas a gauche et n'etant pas arrete
			if (vehicle->m_direction == (m_direction+2)%4 && vehicle->m_nextTurn != LEFT_TURN && !vehicle->m_stop)
			{
				// si ce vehicule est dans la zone directement en face ou dans la zone juste derriere celle en face
				if ((nextZone(X,vehicle->m_direction,vehicle->m_zoneX) == nextZone(X,m_direction,m_zoneX)
					&& nextZone(Y,vehicle->m_direction,vehicle->m_zoneY) == nextZone(Y,m_direction,m_zoneY))
					|| (nextZone(X,vehicle->m_direction,nextZone(X,vehicle->m_direction,vehicle->m_zoneX)) == nextZone(X,m_direction,m_zoneX)
					&& nextZone(Y,vehicle->m_direction,nextZone(Y,vehicle->m_direction,vehicle->m_zoneY)) == nextZone(Y,m_direction,m_zoneY)))
						m_stopRightPriority = true;
					else m_stopRightPriority = false;
			}
		}
	}

	return false;
}


// ============================================================
// Fonction permettant au vehicule de franchir une intersection
// ============================================================
void CVehicle::Turn(CCity * city)
{
	// on teste si on est toujours dans l'intersection
	CIntersection *inter;
	inter = city->GetGraph()->GetInter(m_inter1);
	if ( inter->GetZone()->x == m_zoneX && inter->GetZone()->y == m_zoneY )
	{
		// suivant le virage on change l'angle de rotation
		switch (m_inInter) 
		{
			case RIGHT_TURN:	m_rotation -= 9.6f; break;
			case LEFT_TURN:		m_rotation += 3.2f; break;
			case NO_TURN:		m_inInter = -1; SetRandomNextTurn(city); break;
		}
		if (m_rotation < 0) m_rotation += 360;
		else if (m_rotation >= 360) m_rotation -= 360;
	}
	// on a fini de franchir l'intersection
	else {
		// Maj des variables
		m_rotation = m_tabRot[m_nextDirection];
		m_direction = m_nextDirection;
		m_inInter = -1;
		// on place correctement le vehicule
		switch (m_direction)
		{
			case X :	m_vPosition.z = city->GetPosition().z + POSZONE(m_zoneY) + 3*ZONE_SIZE/4.0f;
						break;
			case Y :	m_vPosition.x = city->GetPosition().x + POSZONE(m_zoneX) + ZONE_SIZE/4.0f;
						break;
			case MX :	m_vPosition.z = city->GetPosition().z + POSZONE(m_zoneY) + ZONE_SIZE/4.0f;
						break;
			case MY :	m_vPosition.x = city->GetPosition().x + POSZONE(m_zoneX) + 3*ZONE_SIZE/4.0f;
						break;
		}
		
		SetRandomNextTurn(city);
	}
}


// =================================================================
// Change la route (intersection de depart et d'arrivee) du vehicule
// =================================================================
void CVehicle::SetRoad(int inter1, int inter2)
{
	m_inter1 = inter1;
	m_inter2 = inter2;
}


// =====================================
// Change l'angle de rotation (braquage)
// =====================================
void CVehicle::SetRotation(float rot)
{
	m_rotation = rot;
}


// =======================================
// Retourne l'angle de rotation (braquage)
// =======================================
float CVehicle::GetRotation()
{
	return m_rotation;
}


// ========================================================
// Choisit aleatoirement quel virage va prendre le vehicule
// ========================================================
void CVehicle::SetRandomNextTurn(CCity *city)
{
	CIntersection *inter = city->GetGraph()->GetInter(m_inter2);
	int i = inter->GetPositionRoad(m_inter1);
	// On choisit au hasard ou va tourner la voiture au prochain virage
	int r;
	for (r = (rand() % 3) + 1; inter->GetRoad((i+r)%4) == NO_ROADS && r != 0; r = (rand() % 3) + 1);
	m_nextTurn = r;
	m_nextDirection = (i+m_nextTurn)%4;
}


// ======================================================================
// Fait avancer le vehicule suivant la vitesse et la position de la ville
// ======================================================================
void CVehicle::MoveForward(float speed, CVector positionCity)
{
	float d;
	float ly = 0.0f;
	float lx = 0.0f;
	
	//printf("Rotation = %f\n", m_rotation);
	if (m_rotation >= 0 && m_rotation <= 90) {
		d = m_rotation / 90.0f;
		m_vPosition.x -= speed * d;				lx = -m_length/2.0f * d;
		m_vPosition.z -= speed * (1.0f - d);	ly = -m_length/2.0f * (1.0f - d);
	}
	else if (m_rotation > 90 && m_rotation <= 180) {
		d = (m_rotation - 90.0f) / 90.0f;
		m_vPosition.z += speed * d;				ly = m_length/2.0f * d;
		m_vPosition.x -= speed * (1.0f - d);	lx = -m_length/2.0f * (1.0f - d);
	}
	else if (m_rotation > 180 && m_rotation <= 270) {
		d = (m_rotation - 180.0f) / 90.0f;
		m_vPosition.x += speed * d;				lx = m_length/2.0f * d;
		m_vPosition.z += speed * (1.0f - d);	ly = m_length/2.0f * (1.0f - d);
	}
	else if (m_rotation > 270 && m_rotation <= 360) {
		d = (m_rotation - 270.0f) / 90.0f;
		m_vPosition.z -= speed * d;				ly = -m_length/2.0f * d;
		m_vPosition.x += speed * (1.0f - d);	lx = m_length/2.0f * (1.0f - d);
	}

	m_zoneX = ((int)(m_vPosition.x - positionCity.x)) / (int)ZONE_SIZE;
	m_zoneY = ((int)(m_vPosition.z - positionCity.z)) / (int)ZONE_SIZE;

	m_nextZoneX = ((int)(m_vPosition.x - positionCity.x + lx)) / (int)ZONE_SIZE;
	m_nextZoneY = ((int)(m_vPosition.z - positionCity.z + ly)) / (int)ZONE_SIZE;
	//printf("X = %d Y = %d\n", m_zoneX, m_zoneY);
}


// ===================================
// Retourne la position X sur la ville
// ===================================
int CVehicle::GetZoneX()
{
	return m_zoneX;
}


// ===================================
// Retourne la position Y sur la ville
// ===================================
int CVehicle::GetZoneY()
{
	return m_zoneY;
}


// ====================
// Supprime ce vehicule		a mettre dans CNode mais je sais pas si le destructeur de CVehicle sera appele ???
// ====================
void CVehicle::Del(CCity *city)
{
	CVehicle *tmp;

	tmp = (CVehicle *)this->m_next;

	this->m_next = tmp->m_next; 

	// On met  jour le flot sur la route ou etait la voiture	
	CZone zone = city->GetZone(tmp->m_zoneX, tmp->m_zoneY);
	if (zone.GetType() == ROAD) zone.GetRoad()->RemoveStream(SENS(tmp->m_direction),CAR_STREAM);	
	else
	{
		zone = city->GetZone(nextZone(X,tmp->m_nextDirection,tmp->m_zoneX),nextZone(Y,tmp->m_nextDirection,tmp->m_zoneY));
		if (zone.GetType() == ROAD) zone.GetRoad()->RemoveStream(SENS(tmp->m_nextDirection),CAR_STREAM);	
	}

	delete tmp;
}


