/*
 * 05/28/2003
 *
 * vehicle.h
 * Represente un vehicule
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#ifndef _VEHICLE_H
#define _VEHICLE_H

#include <GL/glut.h>
#include "object.h"
#include "zone.h"
#include "intersection.h"
#include "node.h"
#include "city.h"
#include "timerGLUT.h"
#include <cstdio>


// ###############
// Classe CVehicle
// ###############
class CVehicle : public CObject, public CNode {

public:

	// Nombre de vehicules affiches
	// Doit tre remis a zero apres chaque affichage de frame
	int static NB_DISPLAY;

	// Dessin en cel-shading
	bool static CELSHADING;
	
	// Nombre d'instances de CVehicule
	int static NB;

	// ============
	// Constructeur
	// ============
	CVehicle(id_t id, char * name);
	
	// ===========
	// Destructeur
	// ===========
	~CVehicle();
	
	// ===================
	// Dessine le vehicule
	// ===================
	void Draw(CFrustum * frustum);

	// ===============================================================================
	// Place et oriente le vehicule sur la ville a la position x,y
	// direction : direction logique X, Y, MX, MY
	// inter1, inter2 : route sur laquelle circule le vehicule (de inter1 vers inter2)
	// city : ville
	// ===============================================================================
	void Initialize(unit_t x, unit_t y, int direction, int inter1, int inter2, CCity * city);

	// ========================
	// Fait evoluer le vehicule 
	// ========================
	void Animate(CCity * city, CTimerGLUT *timer);

	// =================================================================
	// Change la route (intersection de depart et d'arrivee) du vehicule
	// =================================================================
	void SetRoad(int inter1, int inter2);

	// =====================================
	// Change l'angle de rotation (braquage)
	// =====================================
	void SetRotation(float rot);

	// =======================================
	// Retourne l'angle de rotation (braquage)
	// =======================================
	float GetRotation();

	// ===================================
	// Retourne la position X sur la ville
	// ===================================
	int GetZoneX();

	// ===================================
	// Retourne la position Y sur la ville
	// ===================================
	int GetZoneY();

	// ===============================
	// Teste si il y a vehicule devant
	// ===============================
	bool VehiculeFront();

	// ====================
	// Supprime ce vehicule		a mettre dans CNode mais je sais pas si le destructeur de CVehicle sera appele ???
	// ====================
	void Del(CCity *city);

private:
	
	// ===========================================
	// Creation des listes d'affichage du vehicule
	// ===========================================
	void CreateDisplayLists();

	// ======================================================================
	// Fait avancer le vehicule suivant la vitesse et la position de la ville
	// ======================================================================
	void MoveForward(float speed, CVector positionCity);

	// ============================================================
	// Fonction permettant au vehicule de franchir une intersection
	// ============================================================
	void Turn(CCity * city);

	// ==========================================================
	// Recupere la coordonnee de la zone precedente d'une voiture
	// ==========================================================
	unit_t CVehicle::Prevzone(int c);

	// ========================================================
	// Choisit aleatoirement quel virage va prendre le vehicule
	// ========================================================
	void SetRandomNextTurn(CCity *city);

	// Tableau contenant la rotation de la voiture en fonction de sa direction
	int m_tabRot[4];

	// Points pour l'affichage du vehicule
	float m_points[16][3];

	// Identifiant des listes d'affichage
	GLuint m_displayList[2];

	// Direction du vehicule par rapport aux vecteurs de base
	//
	//			-Y
	//			|
	//			|
	//	-X ----------- X	
	//			|
	//			|
	//			Y
	//
	int m_direction;

	// Prochaine direction (lorsque le vehicule est dans une intersection)
	int m_nextDirection;

	// Angle de rotation du vehicule
	float m_rotation;

	// Position sur la ville
	int m_zoneX, m_zoneY;

	// Prochaine position sur la ville
	int m_nextZoneX, m_nextZoneY;

	// Route sur laquelle circule le vehicule (de inter1 vers inter2)
	int m_inter1, m_inter2;

	// Vitesse du vehicule
	float m_speed;

	// Arrt du vehicule
	bool m_stop;

	// Arret devant une intersection
	bool m_stopInter;

	// Arret derriere un autre vehicule
	bool m_stopVehicle;

	// Largeur et longueur du vehicule
	float m_width, m_length;
	
	// Permet de savoir si le vehicule est dans une intersection, -1 sinon
	// et permet de savoir comment l'intersection doit etre franchi LEFT_TURN, RIGHT_TURN, NO_TURN
	int m_inInter;

	// Permet de savoir si la prochaine intersection est franchissable
	bool m_nextInter;

	// Prochain virage
	// Permet de savoir ou va tourner la voiture  la prochaine intersection
	int m_nextTurn;

	// Priorite a droite (arret)
	bool m_stopRightPriority;

};

#endif

