/*
 * 05/28/2003
 *
 * zone.cpp
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "zone.h"

// ############
// Classe CZone
// ############


// ============
// Constructeur
// ============
CZone::CZone()
{
	m_type = FREE; x = 0; y = 0;
}


// ==========================================
// Dessine la zone en fonction de sa position
// ==========================================
void CZone::Draw(CFrustum * frustum)
{
	glPushMatrix();
	glTranslatef(POSZONE(x), 0.0, POSZONE(y));

	switch (m_type)
	{
		case FREE : glColor3f(0.47, 0.29, 0.2); break;
		case ROAD : glColor3f(0.5, 0.5, 0.5); break;
		case INTERSECTION : glColor3f(0.75, 0.05, 0.05); break;
		case RESIDENTIAL : glColor3f(0.05, 0.75, 0.05); break;
		case INDUSTRIAL : glColor3f(0.75, 0.75, 0.05); break;
		case COMMERCIAL : glColor3f(0.05, 0.05, 0.75); break;
	}
	
	glBegin(GL_QUADS);
		glVertex3f(0.0, 0.0, 0.0);
		glVertex3f(ZONE_SIZE, 0.0, 0.0);
		glVertex3f(ZONE_SIZE, 0.0, ZONE_SIZE);
		glVertex3f(0.0, 0.0, ZONE_SIZE);
	glEnd();
	glPopMatrix();
}

// =================================================================
// Retourne le type de la zone (RESIDENTIAL, COMMERCIAL, INDUSTRIAL)
// =================================================================
int CZone::GetType()
{
	return m_type;
}

// ===============================================================
// Change le type de la zone (RESIDENTIAL, COMMERCIAL, INDUSTRIAL)
// ===============================================================
void CZone::SetType(int type)
{
	m_type = type;
}


// ===============================
// Retourne l'id de l'intersection
// ===============================
id_t CZone::GetIdInter()
{
	return m_idInter;
}


// =============================
// Change l'id de l'intersection
// =============================
void CZone::SetIdInter(id_t idInter)
{
	m_idInter = idInter;
}


// =================================================================
// Change la route (uniquement dans le cas ou la zone est une route)
// =================================================================
void CZone::SetRoad(CRoad *road)
{
	m_road = road;
}


// ===================================================================
// Retourne la route (uniquement dans le cas ou la zone est une route)
// ===================================================================
CRoad * CZone::GetRoad()
{
	return m_road;
}
