/*
 * 05/28/2003
 *
 * zone.h
 * Represente une zone de la ville
 *
 * Author: Thierry Vouriot (Yri)
 * yeri@fr.st
 * http://www.yeri.fr.st
 * thierry.vouriot@free.fr
 * http://thierry.vouriot.free.fr
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#ifndef _ZONE_H
#define _ZONE_H

#include "global.h"
#include "frustum.h"
#include "road.h"

// ############
// Classe CZone
// ############
class CZone
{
public:
	// Position x,z de la zone dans la ville
	unit_t x;
	unit_t y;

	// ============
	// Constructeur
	// ============
	CZone();

	// ==========================================
	// Dessine la zone en fonction de sa position
	// ==========================================
	void Draw(CFrustum * frustum);

	// =================================================================
	// Retourne le type de la zone (RESIDENTIAL, COMMERCIAL, INDUSTRIAL)
	// =================================================================
	int GetType();

	// ===============================================================
	// Change le type de la zone (RESIDENTIAL, COMMERCIAL, INDUSTRIAL)
	// ===============================================================
	void SetType(int type);

	// ===============================
	// Retourne l'id de l'intersection
	// ===============================
	id_t GetIdInter();

	// =============================
	// Change l'id de l'intersection
	// =============================
	void SetIdInter(id_t idInter);

	// =================================================================
	// Change la route (uniquement dans le cas ou la zone est une route)
	// =================================================================
	void SetRoad(CRoad *road);

	// ===================================================================
	// Retourne la route (uniquement dans le cas ou la zone est une route)
	// ===================================================================
	CRoad * GetRoad();

private:

	// Type de la zone
	int m_type;

	// Id de l'intersection si la zone est une intersection
	id_t m_idInter;

	// Informations sur la route (uniquement dans le cas ou la zone est une route)
	CRoad * m_road;
};

#endif

